Package: boe
Title: Download Data from the 'Bank of England' Statistical Database
Version: 0.3.0
Authors@R: 
    person("Charles", "Coverdale", , "charlesfcoverdale@gmail.com",
           role = c("aut", "cre"))
Description: Provides functions to download and tidy statistical data
    published by the 'Bank of England' <https://www.bankofengland.co.uk>.
    Covers Bank Rate, 'SONIA', gilt yields, exchange rates, mortgage rates,
    mortgage approvals, consumer credit, and money supply. Series are
    fetched from the 'Bank of England Interactive Statistical Database'
    using its CSV endpoint. Data is cached locally between sessions.
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.3
Imports: cli (>= 3.6.0), httr2 (>= 1.0.0), tools
Suggests: testthat (>= 3.0.0), ggplot2, scales, readxl (>= 1.4.0),
        knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://charlescoverdale.github.io/boe/,
        https://github.com/charlescoverdale/boe
BugReports: https://github.com/charlescoverdale/boe/issues
Depends: R (>= 4.1.0)
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-30 14:20:21 UTC; charlescoverdale
Author: Charles Coverdale [aut, cre]
Maintainer: Charles Coverdale <charlesfcoverdale@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-30 14:30:02 UTC
Built: R 4.5.2; ; 2026-05-30 17:23:00 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,boe_tbl)
export(boe_bank_rate)
export(boe_browse)
export(boe_cache_info)
export(boe_consumer_credit)
export(boe_curve)
export(boe_curve_panel)
export(boe_exchange_rate)
export(boe_get)
export(boe_money_supply)
export(boe_mortgage_approvals)
export(boe_mortgage_rates)
export(boe_mpc_decisions)
export(boe_mpc_votes)
export(boe_mpr_forecasts)
export(boe_search)
export(boe_sonia)
export(boe_yield_curve)
export(clear_cache)
export(list_exchange_rates)

# boe

[![CRAN status](https://www.r-pkg.org/badges/version/boe)](https://CRAN.R-project.org/package=boe) [![CRAN downloads](https://cranlogs.r-pkg.org/badges/boe)](https://cran.r-project.org/package=boe) [![Total Downloads](https://cranlogs.r-pkg.org/badges/grand-total/boe)](https://CRAN.R-project.org/package=boe) [![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable) [![License: MIT](https://img.shields.io/badge/License-MIT-blue.svg)](https://opensource.org/licenses/MIT)

An R package for downloading data from the [Bank of England](https://www.bankofengland.co.uk) Statistical Database.

## What is the Bank of England?

The Bank of England is the United Kingdom's central bank. Founded in 1694, it is responsible for setting monetary policy (including Bank Rate), issuing banknotes, supervising the banking system, and maintaining financial stability. Its Monetary Policy Committee meets eight times a year to set the interest rate that ripples through every mortgage, savings account, and bond in the UK economy.

The Bank publishes thousands of statistical time series through its [Interactive Statistical Database](https://www.bankofengland.co.uk/boeapps/database/) - covering interest rates, exchange rates, money and credit, gilt yields, and housing market indicators. This data underpins monetary policy analysis, financial research, and economic journalism in the UK.

## How is this different from existing packages?

The [`bbk`](https://cran.r-project.org/package=bbk) package on CRAN provides a single generic function for Bank of England data (`bbk::boe_data()`), but it is primarily a Bundesbank client - the Bank of England is one of seven central banks it covers, and its BoE support amounts to a raw API wrapper. You still need to know the series codes, and the output requires further processing.

This package is different. It is built specifically for the Bank of England and provides named, documented functions for the series people actually use - `boe_bank_rate()`, `boe_mortgage_rates()`, `boe_yield_curve()`, and so on. You don't need to know that Bank Rate is `IUDBEDR` or that a 2-year fixed mortgage rate is `IUMBV34`. The package handles series codes, date formatting, caching, and error handling internally.

Beyond the IADB wrappers, it also ships:

- `boe_curve()`: the full Anderson-Sleath fitted yield curves at all maturities, with five curve types (nominal, real, implied inflation, OIS, commercial bank liability), both standard and short-end segments (`segment = "short"` for monthly steps from one month to five years), and full historical archive coverage back to 1979 (nominal), 1985 (real), 2000 (BLC), or 2009 (OIS). `boe_curve_panel()` reshapes to a wide panel at chosen pillar maturities for time-series modelling.
- `boe_search()` / `boe_browse()`: a built-in catalogue of wrapped series so you can find codes from R rather than the website.
- A `boe_tbl` S3 class so every returned data frame carries provenance metadata (series codes, date range, frequency, fetch timestamp).

## Why does this package exist?

The data is freely available, but using it programmatically requires knowing obscure series codes, constructing query URLs with a non-standard date format (`DD/Mon/YYYY`), parsing CSV responses with irregular date formats, and handling HTML error pages returned with HTTP 200 status codes. Every analyst who works with this data writes the same boilerplate.

This package replaces all of that with named functions that return clean data frames.

```r
# Without this package
url <- paste0(
  "https://www.bankofengland.co.uk/boeapps/database/",
  "_iadb-fromshowcolumns.asp?csv.x=yes",
  "&SeriesCodes=IUDBEDR&UsingCodes=Y&CSVF=TN",
  "&Datefrom=01/Jan/2020&Dateto=01/Jan/2025"
)
raw <- read.csv(url)
# ... parse dates, rename columns, handle errors ...

# With this package
library(boe)
boe_bank_rate(from = "2020-01-01")
```

---

## Installation

```r
install.packages("boe")

# Or install the development version from GitHub
# install.packages("devtools")
devtools::install_github("charlescoverdale/boe")
```

---

## Functions

**Data access:**

| Function | Description | From | To |
|---|---|---|---|
| `boe_get()` | Fetch any series by BoE series code | Any | Present |
| `boe_bank_rate()` | Official Bank Rate (daily or monthly) | 1975 | Present |
| `boe_sonia()` | SONIA risk-free reference rate (daily, monthly, or annual) | 1997 | Present |
| `boe_yield_curve()` | Nominal and real gilt yields at 5yr, 10yr, 20yr maturities | 1985 | Present |
| `boe_curve()` | Full Anderson-Sleath fitted curves (nominal / real / inflation / OIS / BLC; spot or forward; standard or short-end) at all maturities | 1979 | Present |
| `boe_curve_panel()` | Wide panel of `boe_curve()` at chosen pillar maturities (segment-aware defaults: 0.5y to 20y standard, 0.5y to 5y short) | 1979 | Present |
| `boe_exchange_rate()` | Daily sterling spot rates for 27 currencies | 1975 | Present |
| `boe_mortgage_rates()` | Quoted mortgage rates (2yr/3yr/5yr fixed, SVR) | 1995 | Present |
| `boe_mortgage_approvals()` | Monthly mortgage approvals for house purchase | 1993 | Present |
| `boe_consumer_credit()` | Consumer credit outstanding (total, cards, other) | 1993 | Present |
| `boe_money_supply()` | M4 broad money amounts outstanding | 1982 | Present |

**Monetary policy:**

| Function | Description | From | To |
|---|---|---|---|
| `boe_mpc_decisions()` | MPC rate-change events: date, new rate, change in bps, direction | 1997 | Present |
| `boe_mpc_votes()` | Full MPC voting record, one row per (meeting, member), with dissent flag | 1997 | Present |
| `boe_mpr_forecasts()` | Monetary Policy Report forecast paths (CPI inflation, GDP growth, GDP level, unemployment, Bank Rate) | 2019 | Present |

**Discovery:**

| Function | Description |
|---|---|
| `boe_series` | Exported catalogue of every wrapped series (code, title, category, frequency, unit, start date) |
| `boe_search()` | Keyword search over `boe_series` |
| `boe_browse()` | Filter `boe_series` by category or frequency |
| `list_exchange_rates()` | Currency codes available to `boe_exchange_rate()` |

**Cache:**

| Function | Description |
|---|---|
| `boe_cache_info()` | Report cache directory, file count, total size |
| `clear_cache()` | Delete locally cached data files |

---

## Examples

### What is Bank Rate today?

```r
library(boe)

# Bank Rate since 2000
br <- boe_bank_rate(from = "2000-01-01")
tail(br, 6)
#>         date rate_pct
#>   2026-02-26     3.75
#>   2026-02-27     3.75
#>   2026-03-02     3.75
#>   2026-03-03     3.75
#>   2026-03-04     3.75
#>   2026-03-05     3.75
```

---

### How has sterling moved against other currencies?

```r
# GBP/USD and GBP/EUR
fx <- boe_exchange_rate(c("USD", "EUR"), from = "2024-01-01", to = "2024-01-31")
head(fx, 6)
#>         date currency   rate
#>   2024-01-02      EUR 1.1536
#>   2024-01-03      EUR 1.1580
#>   2024-01-04      EUR 1.1591
#>   2024-01-05      EUR 1.1615
#>   2024-01-08      EUR 1.1623
#>   2024-01-09      EUR 1.1636

# See all 27 available currencies
list_exchange_rates()
```

---

### What are gilt yields doing?

```r
# 10-year nominal gilt yield
yc <- boe_yield_curve(from = "2024-01-01", to = "2024-01-31", maturity = "10yr")
head(yc, 5)
#>         date maturity yield_pct
#>   2024-01-02     10yr    3.7190
#>   2024-01-03     10yr    3.7638
#>   2024-01-04     10yr    3.8006
#>   2024-01-05     10yr    3.8398
#>   2024-01-08     10yr    3.8619

# Full curve: 5yr, 10yr, and 20yr
boe_yield_curve(from = "2024-01-01")

# Real yields
boe_yield_curve(from = "2024-01-01", type = "real", measure = "zero_coupon")
```

---

### The full Anderson-Sleath fitted curve

For the complete yield curve at every published maturity (typically 0.5 years to 25 or 40 years, in 0.5-year steps), use `boe_curve()`. This parses the BoE's published Excel archive and covers five curves: nominal gilt, real (index-linked) gilt, implied inflation (breakeven), overnight index swap (OIS), and the commercial bank liability curve (BLC). Each curve is published in two segments: the standard curve and a separately fitted short end (monthly steps from one month to five years), reached with `segment = "short"`.

```r
# Latest nominal spot curve at all maturities
nc <- boe_curve(curve = "nominal", measure = "spot")
head(nc, 6)
#>         date maturity_years rate_pct
#>   2026-04-01            0.5    3.95
#>   2026-04-01            1.0    4.10
#>   2026-04-01            1.5    4.13
#>   2026-04-01            2.0    4.15
#>   2026-04-01            2.5    4.16
#>   2026-04-01            3.0    4.17

# Implied inflation curve (breakeven inflation)
boe_curve(curve = "inflation", measure = "spot")

# OIS spot curve
boe_curve(curve = "ois", measure = "spot")

# Short end of the OIS forward curve: the market-implied Bank Rate path
# at monthly resolution, one month to five years
boe_curve(curve = "ois", measure = "forward", segment = "short")
```

Requires the `readxl` package (loaded lazily). Reference: Anderson and Sleath (2001), *New estimates of the UK real and nominal yield curves*, Bank of England Working Paper No. 126.

---

### What are mortgage rates right now?

```r
# All mortgage rate types
mr <- boe_mortgage_rates(from = "2023-01-01")

# Latest rates (as of December 2024)
#>   2yr_fixed: 4.60%
#>   3yr_fixed: 4.48%
#>   5yr_fixed: 4.37%
#>   svr:       7.47%
```

---

### How active is the housing market?

```r
# Monthly mortgage approvals - a leading indicator of housing activity
ma <- boe_mortgage_approvals(from = "2019-01-01")
tail(ma, 6)
#>         date approvals
#>   2025-08-31     64588
#>   2025-09-30     65436
#>   2025-10-31     64634
#>   2025-11-30     64018
#>   2025-12-31     61007
#>   2026-01-31     59999
```

---

### How much are households borrowing?

```r
# Total consumer credit outstanding
cc <- boe_consumer_credit(type = "total", from = "2024-01-01")
tail(cc, 6)
#>         date  type amount_gbp_m
#>   2024-01-31 total       476154
#>   2024-02-29 total       479974
#>   2024-03-31 total       484269
#>   2024-04-30 total       490106
#>   2024-05-31 total       494904
#>   2024-06-30 total       498639

# Credit card debt only
boe_consumer_credit(type = "credit_card", from = "2024-01-01")
```

---

### How much money is in the economy?

```r
# M4 amounts outstanding
m4 <- boe_money_supply(from = "2024-01-01")
head(m4, 6)
#>         date amount_gbp_m
#>   2024-01-31      2986264
#>   2024-02-29      2999033
#>   2024-03-31      3025146
#>   2024-04-30      3030412
#>   2024-05-31      3028825
#>   2024-06-30      3044464   # ← £3 trillion
```

---

### What is the risk-free rate?

```r
# SONIA replaced LIBOR as the UK's benchmark interest rate
sonia <- boe_sonia(from = "2024-01-01", to = "2024-01-31")
head(sonia, 6)
#>         date rate_pct
#>   2024-01-02   5.1863
#>   2024-01-03   5.1863
#>   2024-01-04   5.1870
#>   2024-01-05   5.1869
#>   2024-01-08   5.1869
#>   2024-01-09   5.1867

# Monthly or annual average
boe_sonia(from = "2020-01-01", frequency = "monthly")
```

---

### Fetching any series by code

```r
# If you know the BoE series code, use boe_get() directly
# Series codes: https://www.bankofengland.co.uk/boeapps/database/

# Multiple series in one call - Bank Rate vs SONIA
boe_get(c("IUDBEDR", "IUDSOIA"), from = "2024-01-01", to = "2024-01-10")
#>          date    code  value
#>    2024-01-02 IUDBEDR 5.2500
#>    2024-01-03 IUDBEDR 5.2500
#>    2024-01-04 IUDBEDR 5.2500
#>    ...
#>    2024-01-02 IUDSOIA 5.1863
#>    2024-01-03 IUDSOIA 5.1863
#>    2024-01-04 IUDSOIA 5.1870
#>    ...
```

---

### Tracking MPC decisions and votes

```r
# Every Bank Rate change since 1997
decisions <- boe_mpc_decisions()
tail(decisions, 5)
#>         date new_rate_pct prev_rate_pct change_bps direction
#>   2024-08-01         5.00          5.25        -25       cut
#>   2024-11-07         4.75          5.00        -25       cut
#>   2025-02-06         4.50          4.75        -25       cut
#>   2025-08-07         4.25          4.50        -25       cut
#>   2026-02-05         4.00          4.25        -25       cut

# Full voting record: who dissented, and how
votes <- boe_mpc_votes()
recent_dissents <- subset(votes, dissent & date >= as.Date("2024-01-01"))
head(recent_dissents)

# How does Catherine L Mann vote?
mann <- subset(votes, member == "Catherine L Mann")
table(mann$dissent)
```

---

### Forecasts from the Monetary Policy Report

```r
# Latest CPI inflation projections (one row per publication x horizon)
cpi <- boe_mpr_forecasts(series = "cpi_inflation")
head(cpi)
#>         date horizon horizon_date        series value
#>   2026-02-01 2026 Q1   2026-01-01 cpi_inflation   2.7
#>   2026-02-01 2026 Q2   2026-04-01 cpi_inflation   2.6
#>   2026-02-01 2026 Q3   2026-07-01 cpi_inflation   2.5

# All five headline series for the most recent MPR
all <- boe_mpr_forecasts()
unique(all$series)
#>   [1] "bank_rate" "cpi_inflation" "gdp_growth" "gdp_level" "unemployment"
```

Requires the `readxl` package. Note: this targets the post-2025 MPR file format; older releases use a different archive layout.

---

### Searching for a series

```r
# Keyword search across the catalogue
boe_search("mortgage")

# Filter by category and frequency
boe_search(category = "interest_rates", frequency = "daily")

# Browse without a keyword
boe_browse(category = "exchange_rates")

# The full catalogue is exported as a data frame
head(boe_series)
table(boe_series$category)
#>     consumer_credit       exchange_rates       interest_rates
#>                   3                   27                   14
#>     monetary_aggregates    mortgage_market
#>                       2                  6
```

---

### Provenance

Every result from a `boe_*()` function is a `boe_tbl` (a data frame with attached metadata). Printing shows a one-line provenance header, but it behaves like a normal data frame for everything else.

```r
br <- boe_bank_rate(from = "2024-01-01", frequency = "monthly")
br
#> # BoE [boe_bank_rate]: 1 series [IUMABEDR] · 16 obs · 2024-01-01 to 2025-04-30 · freq=monthly
#>         date rate_pct
#>   2024-01-31     5.25
#>   2024-02-29     5.25
#>   ...
```

---

## Caching

All downloads are cached locally in your user cache directory. Subsequent calls return the cached copy instantly - no network request is made.

```r
# Inspect the cache (path, file count, size, range)
boe_cache_info()
#> BoE cache
#> * Path:  /Users/.../R/boe/cache
#> * Files: 12
#> * Size:  6.4 MB
#> * Range: 2026-04-12 09:14:02 to 2026-04-25 11:30:18

# Force a fresh download
boe_bank_rate(from = "2020-01-01", cache = FALSE)

# Remove files older than 7 days
clear_cache(max_age_days = 7)

# Remove all cached files
clear_cache()
```

---

## Related packages

| Package | Description |
|---|---|
| [`ons`](https://github.com/charlescoverdale/ons) | UK Office for National Statistics data |
| [`hmrc`](https://github.com/charlescoverdale/hmrc) | HM Revenue & Customs tax data |
| [`obr`](https://github.com/charlescoverdale/obr) | Office for Budget Responsibility fiscal forecasts |
| [`fred`](https://github.com/charlescoverdale/fred) | US Federal Reserve (FRED) data |
| [`readecb`](https://github.com/charlescoverdale/readecb) | European Central Bank data |
| [`yieldcurves`](https://github.com/charlescoverdale/yieldcurves) | Yield curve fitting (Nelson-Siegel, Svensson) |
| [`mpshock`](https://github.com/charlescoverdale/mpshock) | Monetary policy shock series (US/UK/AU) |
| [`inflationkit`](https://github.com/charlescoverdale/inflationkit) | Inflation analysis (decomposition, persistence, Phillips curve) |
| [`nowcast`](https://github.com/charlescoverdale/nowcast) | Economic nowcasting (bridge, MIDAS, DFM) |

---

## Issues

Please report bugs or requests at <https://github.com/charlescoverdale/boe/issues>.

## Keywords

Bank of England, BoE, interest rates, bank rate, SONIA, yield curve, exchange rates, mortgage rates, consumer credit, money supply, monetary policy, UK economic data, R package

## ----setup, include = FALSE---------------------------------------------------
library(boe)
not_on_cran <- identical(Sys.getenv("NOT_CRAN"), "true")
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 7,
  fig.height = 4.5,
  eval = not_on_cran
)
op <- options(boe.cache_dir = tempfile("boe_vignette_"))

## ----latest-------------------------------------------------------------------
# latest <- boe_curve(curve = "nominal", measure = "spot")
# range(latest$date)
# range(latest$maturity_years)

## ----latest-provenance--------------------------------------------------------
# attr(latest, "boe_query")$source
# attr(latest, "boe_query")$series_codes

## ----hist-10y-----------------------------------------------------------------
# panel <- boe_curve_panel(
#   curve     = "nominal",
#   measure   = "spot",
#   frequency = "monthly",
#   from      = "2000-01-01",
#   maturities = c(2, 5, 10, 20)
# )
# head(panel)

## ----hist-10y-plot, fig.alt = "10-year UK nominal spot rate, 2000 to present"----
# if (requireNamespace("ggplot2", quietly = TRUE)) {
#   ggplot2::ggplot(panel, ggplot2::aes(date, m10)) +
#     ggplot2::geom_line(colour = "#1f77b4") +
#     ggplot2::labs(
#       title    = "UK 10-year nominal spot rate",
#       subtitle = "End of month, Anderson-Sleath fitted",
#       x = NULL, y = "Per cent"
#     ) +
#     ggplot2::theme_minimal()
# }

## ----five-five----------------------------------------------------------------
# inflation_fwd <- boe_curve_panel(
#   curve     = "inflation",
#   measure   = "forward",
#   frequency = "monthly",
#   from      = "2010-01-01",
#   maturities = c(5, 10)
# )
# 
# inflation_fwd$five_y_five_y <- (inflation_fwd$m10 * 10 -
#                                 inflation_fwd$m5  *  5) / 5
# head(inflation_fwd[, c("date", "m5", "m10", "five_y_five_y")])

## ----five-five-plot, fig.alt = "UK 5y5y forward implied inflation, 2010 to present"----
# if (requireNamespace("ggplot2", quietly = TRUE)) {
#   ggplot2::ggplot(inflation_fwd, ggplot2::aes(date, five_y_five_y)) +
#     ggplot2::geom_line(colour = "#d62728") +
#     ggplot2::geom_hline(yintercept = 2.0, linetype = "dashed",
#                         colour = "grey40") +
#     ggplot2::annotate("text", x = max(inflation_fwd$date),
#                       y = 2.0, label = "2% target", hjust = 1, vjust = -0.5,
#                       colour = "grey40", size = 3) +
#     ggplot2::labs(
#       title    = "UK 5y5y forward implied inflation",
#       subtitle = "End of month, derived from the BoE implied-inflation forward curve",
#       x = NULL, y = "Per cent per annum"
#     ) +
#     ggplot2::theme_minimal()
# }

## ----ois-cycle----------------------------------------------------------------
# ois <- boe_curve_panel(
#   curve     = "ois",
#   measure   = "spot",
#   frequency = "monthly",
#   from      = "2020-01-01",
#   maturities = c(0.5, 1, 2, 5)
# )
# 
# mpc <- boe_mpc_decisions(from = "2020-01-01")
# mpc <- data.frame(date = mpc$date, bank_rate = mpc$new_rate_pct)
# 
# merged <- merge(ois, mpc, by = "date", all.x = TRUE)
# merged$bank_rate <- as.numeric(merged$bank_rate)
# 
# # carry the bank rate forward between MPC dates
# for (i in seq_along(merged$bank_rate)) {
#   if (i > 1 && is.na(merged$bank_rate[i])) {
#     merged$bank_rate[i] <- merged$bank_rate[i - 1]
#   }
# }
# tail(merged)

## ----ois-cycle-plot, fig.alt = "UK OIS spot pillars and Bank Rate, 2020 to present"----
# if (requireNamespace("ggplot2", quietly = TRUE)) {
#   long <- data.frame(
#     date  = rep(merged$date, 5),
#     pillar = rep(c("Bank Rate", "6m OIS", "1y OIS", "2y OIS", "5y OIS"),
#                  each = nrow(merged)),
#     rate  = c(merged$bank_rate, merged$m0.5, merged$m1, merged$m2, merged$m5)
#   )
#   long$pillar <- factor(long$pillar,
#                         levels = c("Bank Rate", "6m OIS", "1y OIS",
#                                    "2y OIS", "5y OIS"))
# 
#   ggplot2::ggplot(long, ggplot2::aes(date, rate, colour = pillar)) +
#     ggplot2::geom_line() +
#     ggplot2::labs(
#       title    = "UK OIS spot pillars and Bank Rate, 2020 to present",
#       subtitle = "Tightening cycle visible across all pillars",
#       x = NULL, y = "Per cent", colour = NULL
#     ) +
#     ggplot2::theme_minimal() +
#     ggplot2::theme(legend.position = "bottom")
# }

## ----short-end----------------------------------------------------------------
# short <- boe_curve(curve = "nominal", measure = "spot", segment = "short")
# range(short$maturity_years)   # monthly grid, ~1/12 to 5 years

## ----short-ois----------------------------------------------------------------
# ois_short  <- boe_curve(curve = "ois", measure = "forward", segment = "short")
# latest_day <- ois_short[ois_short$date == max(ois_short$date), ]
# head(latest_day)

## ----short-ois-plot, fig.alt = "UK OIS short-end forward curve on the latest published date"----
# if (requireNamespace("ggplot2", quietly = TRUE)) {
#   ggplot2::ggplot(latest_day, ggplot2::aes(maturity_years, rate_pct)) +
#     ggplot2::geom_line(colour = "#1f77b4") +
#     ggplot2::geom_point(size = 0.8, colour = "#1f77b4") +
#     ggplot2::labs(
#       title    = "UK OIS instantaneous forward curve, short end",
#       subtitle = paste("Market-implied Bank Rate path on",
#                        format(max(ois_short$date), "%d %B %Y")),
#       x = "Horizon (years)", y = "Per cent"
#     ) +
#     ggplot2::theme_minimal()
# }

## ----teardown, include = FALSE------------------------------------------------
# options(op)

#' Download sterling exchange rates
#'
#' Downloads daily spot exchange rates for sterling against major currencies
#' from the Bank of England. Most series available from January 1975.
#'
#' @param currency Character vector. One or more currency codes. Use
#'   [list_exchange_rates()] to see all available currencies. Defaults to
#'   `"USD"`.
#' @param from Date or character (YYYY-MM-DD). Start date. Defaults to
#'   `"1975-01-02"`.
#' @param to Date or character (YYYY-MM-DD). End date. Defaults to today.
#' @param cache Logical. Use cached data if available (default `TRUE`).
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{date}{Date. Observation date.}
#'     \item{currency}{Character. Currency code (e.g. `"USD"`).}
#'     \item{rate}{Numeric. Units of foreign currency per GBP.}
#'   }
#'
#' @source
#' <https://www.bankofengland.co.uk/boeapps/database/>
#'
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' # GBP/USD since 2020
#' boe_exchange_rate("USD", from = "2020-01-01")
#'
#' # Multiple currencies
#' boe_exchange_rate(c("USD", "EUR", "JPY"), from = "2020-01-01")
#' options(op)
#' }
#'
#' @family exchange rates
#' @export
boe_exchange_rate <- function(currency = "USD",
                              from     = "1975-01-02",
                              to       = Sys.Date(),
                              cache    = TRUE) {

  fx_map <- fx_code_map()
  currency <- toupper(currency)

  bad <- setdiff(currency, names(fx_map))
  if (length(bad) > 0) {
    cli::cli_abort(c(
      "Unknown currenc{?y/ies}: {.val {bad}}",
      "i" = "Use {.code list_exchange_rates()} to see available currencies."
    ))
  }

  codes <- fx_map[currency]

  from <- parse_date_arg(from, "from")
  to   <- parse_date_arg(to, "to")

  out <- boe_fetch(codes, from = from, to = to, cache = cache)

  # Map codes back to currency labels
  code_to_ccy <- stats::setNames(names(codes), codes)
  out$currency <- code_to_ccy[out$code]

  result <- data.frame(
    date     = out$date,
    currency = out$currency,
    rate     = out$value,
    stringsAsFactors = FALSE
  )

  result <- result[order(result$currency, result$date), ]
  rownames(result) <- NULL

  cli::cli_progress_done()
  new_boe_tbl(result, query = list(
    series_codes  = unname(codes),
    from          = from,
    to            = to,
    frequency     = "daily",
    function_name = "boe_exchange_rate"
  ))
}


#' List available exchange rate currencies
#'
#' Returns a data frame of currency codes and descriptions available
#' from the Bank of England exchange rate series.
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{currency}{Character. ISO currency code.}
#'     \item{description}{Character. Currency name.}
#'     \item{boe_code}{Character. BoE series code.}
#'   }
#'
#' @examples
#' list_exchange_rates()
#'
#' @family exchange rates
#' @export
list_exchange_rates <- function() {
  codes <- fx_code_map()
  descs <- fx_descriptions()
  data.frame(
    currency    = names(codes),
    description = descs[names(codes)],
    boe_code    = unname(codes),
    stringsAsFactors = FALSE,
    row.names   = NULL
  )
}


#' @noRd
fx_code_map <- function() {
  c(
    "USD" = "XUDLGBD",
    "EUR" = "XUDLERS",
    "JPY" = "XUDLJYS",
    "CHF" = "XUDLSFS",
    "AUD" = "XUDLADS",
    "CAD" = "XUDLCDS",
    "NZD" = "XUDLNDS",
    "SEK" = "XUDLSKS",
    "NOK" = "XUDLNKS",
    "DKK" = "XUDLDKS",
    "CNY" = "XUDLBK89",
    "INR" = "XUDLBK97",
    "HKD" = "XUDLHDS",
    "SGD" = "XUDLSGS",
    "KRW" = "XUDLBK93",
    "ZAR" = "XUDLZRS",
    "TRY" = "XUDLBK95",
    "PLN" = "XUDLBK47",
    "CZK" = "XUDLBK27",
    "HUF" = "XUDLBK35",
    "ILS" = "XUDLBK65",
    "SAR" = "XUDLSRS",
    "THB" = "XUDLBK87",
    "TWD" = "XUDLTWS",
    "BRL" = "XUDLB8KL",
    "MYR" = "XUDLBK83",
    "ERI" = "XUDLBK67"
  )
}


#' @noRd
fx_descriptions <- function() {
  c(
    "USD" = "US Dollar",
    "EUR" = "Euro",
    "JPY" = "Japanese Yen",
    "CHF" = "Swiss Franc",
    "AUD" = "Australian Dollar",
    "CAD" = "Canadian Dollar",
    "NZD" = "New Zealand Dollar",
    "SEK" = "Swedish Krona",
    "NOK" = "Norwegian Krone",
    "DKK" = "Danish Krone",
    "CNY" = "Chinese Yuan",
    "INR" = "Indian Rupee",
    "HKD" = "Hong Kong Dollar",
    "SGD" = "Singapore Dollar",
    "KRW" = "South Korean Won",
    "ZAR" = "South African Rand",
    "TRY" = "Turkish Lira",
    "PLN" = "Polish Zloty",
    "CZK" = "Czech Koruna",
    "HUF" = "Hungarian Forint",
    "ILS" = "Israeli Shekel",
    "SAR" = "Saudi Riyal",
    "THB" = "Thai Baht",
    "TWD" = "Taiwan Dollar",
    "BRL" = "Brazilian Real",
    "MYR" = "Malaysian Ringgit",
    "ERI" = "Sterling Effective Exchange Rate Index"
  )
}

test_that("boe_exchange_rate() rejects unknown currency", {
  expect_error(boe_exchange_rate("XYZ"), "Unknown")
})

test_that("list_exchange_rates() returns expected structure", {
  out <- list_exchange_rates()
  expect_s3_class(out, "data.frame")
  expect_named(out, c("currency", "description", "boe_code"))
  expect_true("USD" %in% out$currency)
  expect_true("EUR" %in% out$currency)
  expect_true(nrow(out) > 20)
})

test_that("boe_exchange_rate() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_exchange_rate("USD", from = "2024-01-01", to = "2024-01-31")

  expect_s3_class(out, "data.frame")
  expect_named(out, c("date", "currency", "rate"))
  expect_true(all(out$currency == "USD"))
  expect_true(nrow(out) > 0)
})

test_that("boe_exchange_rate() handles multiple currencies", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_exchange_rate(c("USD", "EUR"), from = "2024-01-01", to = "2024-01-31")
  expect_true(all(c("USD", "EUR") %in% out$currency))
})

# Anderson-Sleath yield curve archive helpers.
#
# The BoE publishes one zip per (curve, frequency) combination at fixed URLs.
# Each zip contains 3 Excel workbooks split by period (e.g. 1970-2015,
# 2016-2024, 2025-present), each with the same internal layout. Older real
# and OIS curves start later (real ~1985, OIS ~2009), so the archive zips
# may contain only 2 workbooks for those.

#' Yield-curve archive zip URLs (5 curves x 2 frequencies = 10 entries)
#' @noRd
yield_archive_registry <- function() {
  base <- "https://www.bankofengland.co.uk/-/media/boe/files/statistics/yield-curves"
  list(
    daily = list(
      nominal   = sprintf("%s/glcnominalddata.zip",   base),
      real      = sprintf("%s/glcrealddata.zip",      base),
      inflation = sprintf("%s/glcinflationddata.zip", base),
      ois       = sprintf("%s/oisddata.zip",          base),
      blc       = sprintf("%s/blcnomddata.zip",       base)
    ),
    monthly = list(
      nominal   = sprintf("%s/glcnominalmonthedata.zip",   base),
      real      = sprintf("%s/glcrealmonthedata.zip",      base),
      inflation = sprintf("%s/glcinflationmonthedata.zip", base),
      ois       = sprintf("%s/oismonthedata.zip",          base),
      blc       = sprintf("%s/blcnominalmonthedata.zip",   base)
    )
  )
}

#' Look up the archive zip URL for a (curve, frequency) pair
#' @noRd
yield_archive_url <- function(curve, frequency) {
  reg <- yield_archive_registry()
  if (is.null(reg[[frequency]][[curve]])) {
    cli::cli_abort(c(
      "No archive registered for curve={.val {curve}} frequency={.val {frequency}}.",
      "i" = "Valid curves: nominal, real, inflation, ois, blc."
    ))
  }
  reg[[frequency]][[curve]]
}

#' Download a yield-curve archive zip with a long cache TTL.
#'
#' Archive zips are stable historical artefacts; the BoE only revises them
#' when methodology updates land. A 30-day TTL (720 hours) is a reasonable
#' default. The latest-month zip used by `download_yield_zip()` keeps its
#' shorter 24-hour TTL.
#'
#' @noRd
download_yield_archive <- function(curve, frequency,
                                   cache       = TRUE,
                                   cache_ttl_h = 720) {
  url <- yield_archive_url(curve, frequency)

  cache_dir <- boe_cache_dir()
  dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
  cache_file <- file.path(
    cache_dir,
    sprintf("yield-archive-%s-%s.zip", curve, frequency)
  )

  if (cache && file.exists(cache_file)) {
    age_h <- as.numeric(difftime(Sys.time(),
                                 file.info(cache_file)$mtime,
                                 units = "hours"))
    if (age_h < cache_ttl_h) {
      cli::cli_progress_step(
        "Using cached {curve} {frequency} yield-curve archive"
      )
      return(list(path = cache_file, url = url))
    }
  }

  cli::cli_progress_step(
    "Downloading {curve} {frequency} yield-curve archive from Bank of England"
  )

  tryCatch(
    httr2::request(url) |>
      httr2::req_user_agent("boe R package (https://github.com/charlescoverdale/boe)") |>
      httr2::req_timeout(300) |>
      httr2::req_retry(
        max_tries    = 3,
        is_transient = function(resp) httr2::resp_status(resp) %in% c(429L, 503L)
      ) |>
      httr2::req_perform(path = cache_file),
    error = function(e) {
      if (file.exists(cache_file)) unlink(cache_file)
      cli::cli_abort(c(
        "Download failed for {.val {url}}.",
        "i" = "Check your internet connection.",
        "x" = conditionMessage(e)
      ))
    }
  )

  list(path = cache_file, url = url)
}

#' Extract every Excel workbook in an archive zip that matches a curve.
#'
#' Archive zips contain multiple per-period workbooks (e.g. "1970 to 2015",
#' "2016 to 2024", "2025 to present") all with the same layout. We extract
#' them all and let the parser concatenate.
#'
#' @noRd
extract_archive_excels <- function(zip_path, curve, frequency) {
  files_in_zip <- utils::unzip(zip_path, list = TRUE)$Name

  freq_pattern <- if (frequency == "daily") "daily" else "month end"
  curve_pattern <- switch(curve,
    nominal   = "GLC Nominal",
    real      = "GLC Real",
    inflation = "GLC Inflation",
    ois       = "OIS",
    blc       = "BLC Nominal"
  )

  candidates <- files_in_zip[
    grepl(curve_pattern, files_in_zip, fixed = TRUE) &
    grepl(freq_pattern, files_in_zip, ignore.case = TRUE) &
    grepl("\\.xlsx?$", files_in_zip, ignore.case = TRUE)
  ]

  if (length(candidates) == 0L) {
    cli::cli_abort(c(
      "Could not find any {.val {curve}} {.val {frequency}} workbooks in archive.",
      "i" = "Files in archive: {.val {files_in_zip}}"
    ))
  }

  out_dir <- tempfile("boe_yield_archive_")
  dir.create(out_dir, recursive = TRUE)
  utils::unzip(zip_path,
               files     = candidates,
               exdir     = out_dir,
               junkpaths = TRUE)
  file.path(out_dir, basename(candidates))
}

#' Fetch any series from the Bank of England Statistical Database
#'
#' The core data retrieval function. Fetches one or more series by their
#' BoE series codes and returns a tidy data frame. Use this when the
#' convenience functions (e.g. [boe_bank_rate()], [boe_exchange_rate()])
#' do not cover the series you need.
#'
#' Series codes can be found via the Bank of England Interactive
#' Statistical Database at
#' <https://www.bankofengland.co.uk/boeapps/database/>.
#'
#' @param series_codes Character vector of one or more BoE series codes.
#' @param from Date or character (YYYY-MM-DD). Start date. Defaults to
#'   `"1960-01-01"`.
#' @param to Date or character (YYYY-MM-DD). End date. Defaults to today.
#' @param cache Logical. Use cached data if available (default `TRUE`).
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{date}{Date. Observation date.}
#'     \item{code}{Character. BoE series code.}
#'     \item{value}{Numeric. Observation value.}
#'   }
#'
#' @source
#' <https://www.bankofengland.co.uk/boeapps/database/>
#'
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' # Bank Rate since 2000
#' boe_get("IUDBEDR", from = "2000-01-01")
#'
#' # Multiple series
#' boe_get(c("IUDBEDR", "IUDSOIA"), from = "2020-01-01")
#' options(op)
#' }
#'
#' @family data access
#' @export
boe_get <- function(series_codes,
                    from  = "1960-01-01",
                    to    = Sys.Date(),
                    cache = TRUE) {

  if (length(series_codes) == 0 || !is.character(series_codes)) {
    cli::cli_abort("{.arg series_codes} must be a character vector of BoE series codes.")
  }
  if (length(series_codes) > 300) {
    cli::cli_abort("The BoE API supports a maximum of 300 series per request.")
  }

  from <- parse_date_arg(from, "from")
  to   <- parse_date_arg(to, "to")

  out <- boe_fetch(series_codes, from = from, to = to, cache = cache)

  cli::cli_progress_done()
  new_boe_tbl(out, query = list(
    series_codes  = series_codes,
    from          = from,
    to            = to,
    function_name = "boe_get"
  ))
}

test_that("boe_get() rejects empty series_codes", {
  expect_error(boe_get(character(0)), "series_codes")
})

test_that("boe_get() rejects non-character series_codes", {
 expect_error(boe_get(123), "series_codes")
})

test_that("boe_get() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_get("IUDBEDR", from = "2024-01-01", to = "2024-01-31")

  expect_s3_class(out, "data.frame")
  expect_named(out, c("date", "code", "value"))
  expect_s3_class(out$date, "Date")
  expect_type(out$code, "character")
  expect_type(out$value, "double")
  expect_true(nrow(out) > 0)
  expect_true(all(out$code == "IUDBEDR"))
})

test_that("boe_get() handles multiple series", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_get(c("IUDBEDR", "IUDSOIA"), from = "2024-01-01", to = "2024-01-31")

  expect_true(all(c("IUDBEDR", "IUDSOIA") %in% out$code))
})

#' Monetary Policy Committee voting history
#'
#' Downloads the Bank of England's published MPC voting record and
#' returns it in long format. Each row is one (meeting, member) pair
#' showing the rate the member voted for and whether that constituted
#' a dissent from the committee's decision.
#'
#' Coverage runs from the first MPC meeting in June 1997 through the
#' most recent published minutes. Both current and past committee
#' members are included.
#'
#' @param cache Logical. Use cached download if less than 24 hours old
#'   (default `TRUE`). The voting workbook is small (~110 KB).
#'
#' @return A `boe_tbl` data frame with columns:
#'   \describe{
#'     \item{date}{Date. Meeting date.}
#'     \item{member}{Character. MPC member name.}
#'     \item{member_vote_pct}{Numeric. The Bank Rate the member voted
#'       for (percent).}
#'     \item{decision_pct}{Numeric. The committee's decision (percent).}
#'     \item{dissent}{Logical. `TRUE` if the member's vote differed from
#'       the committee decision.}
#'   }
#'
#' @details Requires the \pkg{readxl} package.
#'
#' @source <https://www.bankofengland.co.uk/monetary-policy>
#'
#' @examples
#' \donttest{
#' if (requireNamespace("readxl", quietly = TRUE)) {
#'   op <- options(boe.cache_dir = tempdir())
#'   votes <- boe_mpc_votes()
#'
#'   # Recent dissents
#'   recent <- subset(votes, dissent & date >= as.Date("2024-01-01"))
#'   head(recent)
#'
#'   options(op)
#' }
#' }
#'
#' @family monetary policy
#' @seealso [boe_mpc_decisions()]
#' @export
boe_mpc_votes <- function(cache = TRUE) {

  if (!requireNamespace("readxl", quietly = TRUE)) {
    cli::cli_abort(c(
      "Package {.pkg readxl} is required to parse the MPC voting file.",
      "i" = "Install it with {.code install.packages(\"readxl\")}."
    ))
  }

  cache_file <- file.path(boe_cache_dir(), "mpcvoting.xlsx")
  dir.create(dirname(cache_file), recursive = TRUE, showWarnings = FALSE)

  fresh <- cache &&
    file.exists(cache_file) &&
    as.numeric(difftime(Sys.time(), file.info(cache_file)$mtime,
                        units = "hours")) < 24

  if (fresh) {
    cli::cli_progress_step("Using cached MPC voting record")
  } else {
    cli::cli_progress_step("Downloading MPC voting record from Bank of England")
    url <- mpcvoting_url()
    tryCatch(
      httr2::request(url) |>
        httr2::req_user_agent("boe R package (https://github.com/charlescoverdale/boe)") |>
        httr2::req_timeout(60) |>
        httr2::req_retry(
          max_tries    = 3,
          is_transient = function(resp) httr2::resp_status(resp) %in% c(429L, 503L)
        ) |>
        httr2::req_perform(path = cache_file),
      error = function(e) {
        if (file.exists(cache_file)) unlink(cache_file)
        cli::cli_abort(c(
          "Download failed.",
          "i" = "Check your internet connection.",
          "x" = conditionMessage(e)
        ))
      }
    )
  }

  parse_mpcvoting(cache_file)
}


#' @noRd
mpcvoting_url <- function() {
  "https://www.bankofengland.co.uk/-/media/boe/files/monetary-policy-summary-and-minutes/mpcvoting.xlsx"
}


#' Parse the BoE MPC voting workbook
#'
#' Sheet "Bank Rate Decisions" lays out one row per meeting (from row
#' 11 downwards) and one column per member (in row 3). Column 2 holds
#' an Excel serial meeting date and column 3 holds the rate as a
#' decimal (e.g. 0.04 for 4.00%).
#'
#' @noRd
parse_mpcvoting <- function(path) {
  raw <- suppressMessages(readxl::read_excel(
    path, sheet = "Bank Rate Decisions", col_names = FALSE
  ))
  raw <- as.data.frame(raw, stringsAsFactors = FALSE)

  if (nrow(raw) < 11L || ncol(raw) < 4L) {
    cli::cli_abort("MPC voting sheet is too small to parse.")
  }

  header  <- as.character(unlist(raw[3L, ]))
  is_name <- !is.na(header) &
             nzchar(header) &
             !header %in% c("Current members", "Past members")
  member_cols  <- which(is_name)
  member_names <- header[member_cols]

  if (length(member_cols) == 0L) {
    cli::cli_abort("No MPC member names found on row 3 of the voting sheet.")
  }

  body    <- raw[11L:nrow(raw), , drop = FALSE]
  serials <- suppressWarnings(as.numeric(body[[2L]]))
  rates   <- suppressWarnings(as.numeric(body[[3L]]))
  good    <- !is.na(serials) & !is.na(rates) & serials > 30000
  body    <- body[good, , drop = FALSE]
  serials <- serials[good]
  rates   <- rates[good]

  if (nrow(body) == 0L) {
    cli::cli_abort("No meeting rows found in the MPC voting sheet.")
  }

  meeting_dates <- as.Date(serials, origin = "1899-12-30")
  decision_pct  <- rates * 100

  vote_mat <- suppressWarnings(matrix(
    as.numeric(as.matrix(body[, member_cols, drop = FALSE])),
    nrow = nrow(body),
    ncol = length(member_cols)
  )) * 100

  long <- data.frame(
    date            = rep(meeting_dates, times = ncol(vote_mat)),
    member          = rep(member_names,  each  = nrow(vote_mat)),
    member_vote_pct = as.numeric(vote_mat),
    decision_pct    = rep(decision_pct,  times = ncol(vote_mat)),
    stringsAsFactors = FALSE
  )
  long$dissent <- !is.na(long$member_vote_pct) &
    abs(long$member_vote_pct - long$decision_pct) > 1e-6
  long <- long[!is.na(long$member_vote_pct), , drop = FALSE]
  long <- long[order(long$date, long$member), , drop = FALSE]
  rownames(long) <- NULL

  cli::cli_progress_done()
  new_boe_tbl(long, query = list(
    series_codes  = "MPCVOTING_BANKRATE",
    from          = min(long$date),
    to            = max(long$date),
    frequency     = "meeting",
    function_name = "boe_mpc_votes"
  ))
}

# Tests for boe_mpc_votes() (parses BoE mpcvoting.xlsx).

test_that("boe_mpc_votes returns the expected schema", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("readxl")

  op <- options(boe.cache_dir = tempfile("boe_mpc_votes_"))
  on.exit(options(op), add = TRUE)

  out <- boe_mpc_votes()

  expect_s3_class(out, "boe_tbl")
  expected_cols <- c("date", "member", "member_vote_pct",
                     "decision_pct", "dissent")
  expect_true(all(expected_cols %in% names(out)))
  expect_gt(nrow(out), 100L)
  expect_true(all(is.finite(out$member_vote_pct)))
  expect_true(all(is.finite(out$decision_pct)))
  expect_true(is.logical(out$dissent))
  expect_true(all(out$date >= as.Date("1997-06-01")))

  q <- attr(out, "boe_query")
  expect_equal(q$function_name, "boe_mpc_votes")
})

#' Search the BoE series catalogue
#'
#' Filters the [boe_series] catalogue by keyword and optional category
#' and frequency. Useful for finding a series code without leaving R.
#' Equivalent to a `grepl` against the title and code columns.
#'
#' @param query Character. Keyword(s) to match against the `title` and
#'   `code` columns. Case-insensitive substring match. If `NULL`,
#'   returns all rows (subject to other filters).
#' @param category Character. Optional filter on the `category` column,
#'   one or more of `"interest_rates"`, `"exchange_rates"`,
#'   `"mortgage_market"`, `"consumer_credit"`, `"monetary_aggregates"`.
#' @param frequency Character. Optional filter on the `frequency`
#'   column, one or more of `"daily"`, `"monthly"`, `"annual"`.
#'
#' @return A data frame with the same columns as [boe_series],
#'   restricted to matching rows.
#'
#' @examples
#' # Find mortgage-related series
#' boe_search("mortgage")
#'
#' # All daily interest-rate series
#' boe_search(category = "interest_rates", frequency = "daily")
#'
#' # Locate the Bank Rate code
#' boe_search("bank rate")
#'
#' @family discovery
#' @seealso [boe_browse()], [boe_series]
#' @export
boe_search <- function(query     = NULL,
                       category  = NULL,
                       frequency = NULL) {

  out <- get("boe_series", envir = asNamespace("boe"))

  if (!is.null(query)) {
    if (!is.character(query)) {
      cli::cli_abort("{.arg query} must be a character string.")
    }
    q <- tolower(paste(query, collapse = " "))
    hay <- tolower(paste(out$title, out$code))
    out <- out[grepl(q, hay, fixed = TRUE), , drop = FALSE]
  }
  if (!is.null(category)) {
    out <- out[out$category %in% category, , drop = FALSE]
  }
  if (!is.null(frequency)) {
    out <- out[out$frequency %in% frequency, , drop = FALSE]
  }

  rownames(out) <- NULL
  out
}


#' Browse the BoE series catalogue
#'
#' Returns the catalogue with optional category or frequency filters.
#' Equivalent to `boe_search(query = NULL, category, frequency)` but
#' framed as a browse / inspect action rather than a keyword search.
#'
#' @inheritParams boe_search
#' @return A data frame with the same columns as [boe_series].
#'
#' @examples
#' # The whole catalogue
#' nrow(boe_browse())
#'
#' # All exchange rate series
#' boe_browse(category = "exchange_rates")
#'
#' # All monthly series
#' boe_browse(frequency = "monthly")
#'
#' @family discovery
#' @seealso [boe_search()], [boe_series]
#' @export
boe_browse <- function(category = NULL, frequency = NULL) {
  boe_search(query = NULL, category = category, frequency = frequency)
}

# Tests for boe_search() and boe_browse() catalogue helpers.

test_that("boe_series catalogue has required columns and shape", {
  expect_s3_class(boe_series, "data.frame")
  expected_cols <- c("code", "title", "category", "frequency",
                     "unit", "start_date", "seasonal_adjustment", "helper")
  expect_true(all(expected_cols %in% names(boe_series)))
  expect_gt(nrow(boe_series), 40L)
  expect_true(all(nzchar(boe_series$code)))
  expect_true(all(nzchar(boe_series$title)))
})

test_that("boe_search() with no filters returns the full catalogue", {
  expect_equal(nrow(boe_search()), nrow(boe_series))
})

test_that("boe_search() filters on a keyword", {
  hits <- boe_search("mortgage")
  expect_gt(nrow(hits), 0L)
  expect_true(all(grepl("mortgage", hits$title, ignore.case = TRUE) |
                  grepl("mortgage", hits$code,  ignore.case = TRUE)))
})

test_that("boe_search() is case-insensitive", {
  expect_equal(nrow(boe_search("BANK RATE")),
               nrow(boe_search("bank rate")))
})

test_that("boe_search() filters on category", {
  fx <- boe_search(category = "exchange_rates")
  expect_true(all(fx$category == "exchange_rates"))
  expect_gt(nrow(fx), 20L)
})

test_that("boe_search() filters on frequency", {
  daily <- boe_search(frequency = "daily")
  expect_true(all(daily$frequency == "daily"))
})

test_that("boe_search() composes filters", {
  hits <- boe_search("yield", category = "interest_rates", frequency = "daily")
  expect_true(all(hits$category == "interest_rates"))
  expect_true(all(hits$frequency == "daily"))
})

test_that("boe_search() rejects non-character query", {
  expect_error(boe_search(query = 1L), "character")
})

test_that("boe_search() returns zero rows on no match", {
  expect_equal(nrow(boe_search("definitely_not_a_real_series_xyz")), 0L)
})

test_that("boe_browse() equals boe_search() with no query", {
  expect_equal(boe_browse(), boe_search())
  expect_equal(boe_browse(category = "consumer_credit"),
               boe_search(category = "consumer_credit"))
})

#' Download mortgage approvals for house purchase
#'
#' Downloads the monthly count of mortgage approvals for house purchase,
#' a widely watched leading indicator of housing market activity.
#' Available from April 1993.
#'
#' @param from Date or character (YYYY-MM-DD). Start date. Defaults to
#'   `"1993-04-01"`.
#' @param to Date or character (YYYY-MM-DD). End date. Defaults to today.
#' @param seasonally_adjusted Logical. Return seasonally adjusted series
#'   (default `TRUE`) or non-seasonally adjusted (`FALSE`).
#' @param cache Logical. Use cached data if available (default `TRUE`).
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{date}{Date. End of month.}
#'     \item{approvals}{Numeric. Number of mortgage approvals.}
#'   }
#'
#' @source
#' <https://www.bankofengland.co.uk/boeapps/database/>
#'
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' boe_mortgage_approvals(from = "2015-01-01")
#' options(op)
#' }
#'
#' @family credit and housing
#' @export
boe_mortgage_approvals <- function(from                 = "1993-04-01",
                                   to                   = Sys.Date(),
                                   seasonally_adjusted  = TRUE,
                                   cache                = TRUE) {

  code <- if (seasonally_adjusted) "LPMVTVX" else "LPMVTVU"

  from <- parse_date_arg(from, "from")
  to   <- parse_date_arg(to, "to")

  out <- boe_fetch(code, from = from, to = to, cache = cache)

  result <- data.frame(
    date      = out$date,
    approvals = out$value,
    stringsAsFactors = FALSE
  )

  cli::cli_progress_done()
  new_boe_tbl(result, query = list(
    series_codes  = code,
    from          = from,
    to            = to,
    frequency     = "monthly",
    function_name = "boe_mortgage_approvals"
  ))
}

#' Download UK gilt yields
#'
#' Downloads nominal or real gilt yields at specified maturities from the
#' Bank of England yield curve data. Nominal par yields are available
#' daily from late 1993; real zero-coupon yields from 1985.
#'
#' @param from Date or character (YYYY-MM-DD). Start date.
#' @param to Date or character (YYYY-MM-DD). End date. Defaults to today.
#' @param maturity Character vector. One or more of `"5yr"`, `"10yr"`,
#'   `"20yr"`. Defaults to all three.
#' @param type Character. `"nominal"` (default) or `"real"`.
#' @param measure Character. `"par_yield"` (default, nominal only) or
#'   `"zero_coupon"`.
#' @param cache Logical. Use cached data if available (default `TRUE`).
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{date}{Date. Observation date.}
#'     \item{maturity}{Character. Maturity label (e.g. `"5yr"`).}
#'     \item{yield_pct}{Numeric. Yield (percent).}
#'   }
#'
#' @source
#' <https://www.bankofengland.co.uk/boeapps/database/>
#'
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' # 10-year nominal gilt yield since 2020
#' boe_yield_curve(from = "2020-01-01", maturity = "10yr")
#'
#' # Full nominal curve
#' boe_yield_curve(from = "2020-01-01")
#'
#' # Real yields
#' boe_yield_curve(from = "2020-01-01", type = "real", measure = "zero_coupon")
#' options(op)
#' }
#'
#' @family interest rates
#' @export
boe_yield_curve <- function(from     = "2000-01-01",
                            to       = Sys.Date(),
                            maturity = c("5yr", "10yr", "20yr"),
                            type     = c("nominal", "real"),
                            measure  = c("par_yield", "zero_coupon"),
                            cache    = TRUE) {

  type    <- match.arg(type)
  measure <- match.arg(measure)
  maturity <- match.arg(maturity, several.ok = TRUE)

  if (type == "real" && measure == "par_yield") {
    cli::cli_abort("Real par yields are not available. Use {.code measure = \"zero_coupon\"} for real yields.")
  }

  # Build series code lookup
  code_map <- list(
    nominal_par_yield = c("5yr" = "IUDSNPY", "10yr" = "IUDMNPY", "20yr" = "IUDLNPY"),
    nominal_zero_coupon = c("5yr" = "IUDSNZC", "10yr" = "IUDMNZC", "20yr" = "IUDLNZC"),
    real_zero_coupon = c("5yr" = "IUDSRZC", "10yr" = "IUDMRZC", "20yr" = "IUDLRZC")
  )

  key   <- paste0(type, "_", measure)
  codes <- code_map[[key]][maturity]

  from <- parse_date_arg(from, "from")
  to   <- parse_date_arg(to, "to")

  out <- boe_fetch(codes, from = from, to = to, cache = cache)

  # Map codes back to maturity labels
  code_to_maturity <- stats::setNames(names(codes), codes)
  out$maturity <- code_to_maturity[out$code]

  result <- data.frame(
    date      = out$date,
    maturity  = out$maturity,
    yield_pct = out$value,
    stringsAsFactors = FALSE
  )

  result <- result[order(result$date, result$maturity), ]
  rownames(result) <- NULL

  cli::cli_progress_done()
  new_boe_tbl(result, query = list(
    series_codes  = unname(codes),
    from          = from,
    to            = to,
    frequency     = "daily",
    function_name = "boe_yield_curve"
  ))
}

test_that("boe_yield_curve() rejects real par_yield", {
  expect_error(
    boe_yield_curve(type = "real", measure = "par_yield"),
    "par yields"
  )
})

test_that("boe_yield_curve() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_yield_curve(from = "2024-01-01", to = "2024-01-31", maturity = "10yr")

  expect_s3_class(out, "data.frame")
  expect_named(out, c("date", "maturity", "yield_pct"))
  expect_true(all(out$maturity == "10yr"))
  expect_true(nrow(out) > 0)
})

test_that("boe_yield_curve() handles multiple maturities", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_yield_curve(from = "2024-01-01", to = "2024-01-31")
  expect_true(all(c("5yr", "10yr", "20yr") %in% out$maturity))
})

#' Download SONIA interest rate
#'
#' Downloads the Sterling Overnight Index Average (SONIA), the risk-free
#' reference rate for sterling markets. Available daily from January 1997.
#'
#' @param from Date or character (YYYY-MM-DD). Start date. Defaults to
#'   `"1997-01-02"`.
#' @param to Date or character (YYYY-MM-DD). End date. Defaults to today.
#' @param frequency Character. One of `"daily"` (default), `"monthly"`
#'   (monthly average), or `"annual"` (annual average).
#' @param cache Logical. Use cached data if available (default `TRUE`).
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{date}{Date. Observation date.}
#'     \item{rate_pct}{Numeric. SONIA rate (percent).}
#'   }
#'
#' @source
#' <https://www.bankofengland.co.uk/boeapps/database/>
#'
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' boe_sonia(from = "2020-01-01")
#' options(op)
#' }
#'
#' @family interest rates
#' @export
boe_sonia <- function(from      = "1997-01-02",
                      to        = Sys.Date(),
                      frequency = c("daily", "monthly", "annual"),
                      cache     = TRUE) {

  frequency <- match.arg(frequency)
  code <- switch(frequency,
    daily   = "IUDSOIA",
    monthly = "IUMASOIA",
    annual  = "IUAASOIA"
  )

  from <- parse_date_arg(from, "from")
  to   <- parse_date_arg(to, "to")

  out <- boe_fetch(code, from = from, to = to, cache = cache)

  result <- data.frame(
    date     = out$date,
    rate_pct = out$value,
    stringsAsFactors = FALSE
  )

  cli::cli_progress_done()
  new_boe_tbl(result, query = list(
    series_codes  = code,
    from          = from,
    to            = to,
    frequency     = frequency,
    function_name = "boe_sonia"
  ))
}

test_that("boe_sonia() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_sonia(from = "2024-01-01", to = "2024-01-31")

  expect_s3_class(out, "data.frame")
  expect_named(out, c("date", "rate_pct"))
  expect_s3_class(out$date, "Date")
  expect_type(out$rate_pct, "double")
  expect_true(nrow(out) > 0)
})

test_that("boe_sonia() monthly frequency works", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_sonia(from = "2024-01-01", to = "2024-06-30", frequency = "monthly")
  expect_true(nrow(out) >= 1)
  expect_named(out, c("date", "rate_pct"))
})

test_that("boe_sonia() annual frequency works", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_sonia(from = "2020-01-01", to = "2024-12-31", frequency = "annual")
  expect_true(nrow(out) >= 1)
  expect_named(out, c("date", "rate_pct"))
})

test_that("boe_sonia() rejects invalid frequency", {
  expect_error(boe_sonia(frequency = "weekly"))
})

#' @keywords internal
#' @concept Bank of England
#' @concept interest rates
#' @concept monetary policy
#' @concept SONIA
#' @concept yield curve
#' @concept exchange rates
#' @concept mortgage rates
#' @concept money supply
#' @concept UK central bank
"_PACKAGE"

#' Monetary Policy Report forecast data
#'
#' Downloads the Bank of England's Monetary Policy Report (MPR) and
#' parses headline projections from the Projections Databank workbook.
#' Returns a long-format data frame where each row is one (publication
#' date, forecast horizon, series) triple.
#'
#' Coverage runs quarterly from November 2019 (when the report was
#' renamed from Inflation Report) to the latest published release.
#'
#' @param series Character vector. One or more of:
#'   `"cpi_inflation"`, `"gdp_growth"`, `"gdp_level"`,
#'   `"unemployment"`, `"bank_rate"`. Defaults to all five.
#' @param month Character. Publication month of the report, e.g.
#'   `"february"` or `"may"`. The report is published roughly quarterly,
#'   but the exact month drifts between years (for example, the second
#'   2026 report appeared in April, not May), so any month name is
#'   accepted and its existence is verified against the Bank's website.
#'   Supply with `year`. If both are `NULL`, the most recent compatible
#'   release is selected automatically.
#' @param year Integer. MPR year, 2019 or later. Supply with `month`.
#'   If both are `NULL`, the most recent compatible release is selected
#'   automatically.
#' @param cache Logical. Use cached download if available (default
#'   `TRUE`). Older releases never change so the cache never expires;
#'   the latest release is refreshed if older than 24 hours.
#'
#' @return A `boe_tbl` data frame with columns:
#'   \describe{
#'     \item{date}{Date. Publication date of the MPR (start of quarter
#'       the report covers).}
#'     \item{horizon}{Character. Forecast horizon label (e.g. `"2026 Q1"`).}
#'     \item{horizon_date}{Date. Start of the forecast quarter.}
#'     \item{series}{Character. Series identifier (e.g. `"cpi_inflation"`).}
#'     \item{value}{Numeric. Forecast value (percent for rates and
#'       growth; index for `gdp_level`).}
#'   }
#'
#' @details
#' Requires the \pkg{readxl} package. The MPR is published as a zip
#' archive containing a Projections Databank workbook plus chart data
#' and slides; this function only reads the projection sheets.
#'
#' Each row of a projection sheet is one MPR publication; columns are
#' forecast quarters. The same publication therefore contributes
#' multiple rows here, one per forecast horizon.
#'
#' @source <https://www.bankofengland.co.uk/monetary-policy>
#'
#' @section Release format and automatic fallback:
#' From the April 2026 report the Bank moved to a scenario-based
#' "Scenario Projections Databank" with a transposed layout (following
#' the Bernanke review of forecasting). That format is not parsed by
#' this function yet. When automatic selection encounters such a
#' release it skips it, falls back to the most recent compatible release
#' (the classic "Projections Databank" workbook), and warns. Requesting
#' a scenario-format release explicitly via `month`/`year` raises a
#' clear error. Pre-2020 MPRs that predate the single "Projections
#' Databank" workbook may also error.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("readxl", quietly = TRUE)) {
#'   op <- options(boe.cache_dir = tempdir())
#'
#'   # Latest CPI inflation projections
#'   cpi <- boe_mpr_forecasts(series = "cpi_inflation")
#'   head(cpi)
#'
#'   options(op)
#' }
#' }
#'
#' @family monetary policy
#' @seealso [boe_mpc_decisions()], [boe_mpc_votes()]
#' @export
boe_mpr_forecasts <- function(series = c("cpi_inflation", "gdp_growth",
                                         "gdp_level", "unemployment",
                                         "bank_rate"),
                              month  = NULL,
                              year   = NULL,
                              cache  = TRUE) {

  if (!requireNamespace("readxl", quietly = TRUE)) {
    cli::cli_abort(c(
      "Package {.pkg readxl} is required to parse MPR data.",
      "i" = "Install it with {.code install.packages(\"readxl\")}."
    ))
  }

  series_choices <- eval(formals()$series)
  if (missing(series)) {
    series <- series_choices
  } else {
    series <- match.arg(series, choices = series_choices, several.ok = TRUE)
  }

  if (is.null(month) && is.null(year)) {
    picked   <- pick_mpr_release(cache = cache)
    release  <- picked$release
    zip_path <- picked$zip_path
  } else {
    release  <- resolve_mpr_release(month = month, year = year)
    zip_path <- download_mpr_zip(release$month, release$year, cache = cache)
    if (!mpr_zip_is_old_format(zip_path)) {
      rel_label <- sprintf("%s %d", tools::toTitleCase(release$month),
                           release$year)
      cli::cli_abort(c(
        "The {rel_label} MPR uses the Bank of England's new scenario-based format.",
        "i" = "This format is not parsed by {.fn boe_mpr_forecasts} yet.",
        "i" = "Request an earlier release (February 2026 or before), or omit {.arg month} and {.arg year} to auto-select the latest compatible release."
      ))
    }
  }
  xls_path <- extract_projections_databank(zip_path)

  parts <- lapply(series, function(s) {
    parse_projection_sheet(xls_path,
                           sheet_name  = mpr_sheet_for(s),
                           series_name = s)
  })
  out <- do.call(rbind, parts)
  out <- out[order(out$date, out$horizon_date, out$series), , drop = FALSE]
  rownames(out) <- NULL

  cli::cli_progress_done()
  new_boe_tbl(out, query = list(
    series_codes  = paste0("MPR_", toupper(series)),
    from          = min(out$date),
    to            = max(out$date),
    frequency     = "quarterly",
    function_name = "boe_mpr_forecasts",
    vintage       = sprintf("%s %d", release$month, release$year)
  ))
}


#' Map a friendly series name to the Projections Databank sheet name
#' @noRd
mpr_sheet_for <- function(series) {
  switch(series,
    cpi_inflation = "1. CPI inflation",
    gdp_growth    = "2. GDP growth",
    gdp_level     = "3. GDP level",
    unemployment  = "4. Unemployment",
    bank_rate     = "38. Bank Rate",
    cli::cli_abort("Unknown MPR series {.val {series}}.")
  )
}


#' Validate an explicitly requested (month, year) MPR release
#'
#' Used only when the caller supplies `month`/`year`. The publication
#' schedule drifts between years, so any valid month name is accepted;
#' whether that release actually exists is verified at download time.
#' @noRd
resolve_mpr_release <- function(month = NULL, year = NULL,
                                today = Sys.Date()) {
  if (is.null(month) && is.null(year)) {
    return(mpr_release_candidates(today)[[1L]])
  }
  if (is.null(month) || is.null(year)) {
    cli::cli_abort("Specify both {.arg month} and {.arg year}, or neither.")
  }
  month <- tolower(as.character(month))
  if (!month %in% tolower(month.name)) {
    cli::cli_abort('{.arg month} must be a month name, e.g. {.val february} or {.val may}.')
  }
  year <- as.integer(year)
  if (year < 2019L) {
    cli::cli_abort("MPR coverage starts at November 2019. For earlier dates use the legacy Inflation Report.")
  }
  list(month = month, year = year)
}


#' Candidate MPR releases, most recent first
#'
#' The report is published roughly quarterly but the exact publication
#' month drifts between years, so rather than assume a fixed Feb/May/
#' Aug/Nov calendar we enumerate the last `n_months` months and let the
#' caller probe which ones actually exist. Eight months spans more than
#' two quarterly cycles, enough to find the latest release even when the
#' two most recent ones are unpublished or in an unsupported format.
#' @noRd
mpr_release_candidates <- function(today = Sys.Date(), n_months = 8L) {
  yr     <- as.integer(format(today, "%Y"))
  mo     <- as.integer(format(today, "%m"))
  months <- tolower(month.name)
  lapply(seq_len(n_months) - 1L, function(k) {
    m <- mo - k
    y <- yr
    while (m <= 0L) {
      m <- m + 12L
      y <- y - 1L
    }
    list(month = months[m], year = y)
  })
}


#' Candidate zip URLs for a release, most likely first
#'
#' The data archive filename drifted from "chart-slides-and-data"
#' (February 2025 and earlier) to "charts-slides-and-data" (May 2025
#' onwards). Both variants are returned so the caller can probe each.
#' @noRd
mpr_zip_urls <- function(month, year) {
  base <- sprintf(
    "https://www.bankofengland.co.uk/-/media/boe/files/monetary-policy-report/%d/%s/mpr-%s-%d-",
    year, month, month, year
  )
  variants <- c("charts-slides-and-data", "chart-slides-and-data")
  if (year < 2025L) variants <- rev(variants)
  paste0(base, variants, ".zip")
}


#' Primary (best-guess) zip URL for a release
#' @noRd
mpr_zip_url <- function(month, year) {
  mpr_zip_urls(month, year)[[1L]]
}


#' Does a URL resolve (HTTP < 400)? Returns FALSE on any network error.
#' @noRd
url_exists_boe <- function(url) {
  tryCatch({
    resp <- httr2::request(url) |>
      httr2::req_user_agent("boe R package (https://github.com/charlescoverdale/boe)") |>
      httr2::req_method("HEAD") |>
      httr2::req_timeout(30) |>
      httr2::req_error(is_error = function(resp) FALSE) |>
      httr2::req_perform()
    httr2::resp_status(resp) < 400L
  }, error = function(e) FALSE)
}


#' First existing zip URL for a release, or NULL if none resolve
#' @noRd
mpr_resolve_url <- function(month, year) {
  for (u in mpr_zip_urls(month, year)) {
    if (url_exists_boe(u)) return(u)
  }
  NULL
}


#' Select the most recent compatible (parseable) MPR release
#'
#' Walks back through candidate releases, newest first, probing each
#' URL. The first release that both exists and uses the classic
#' "Projections Databank" workbook is returned. Newer releases that use
#' the unsupported scenario format are skipped with a warning. Errors
#' only if no compatible release resolves in the lookback window.
#' @noRd
pick_mpr_release <- function(today = Sys.Date(), cache = TRUE,
                             max_lookback = 8L) {
  candidates <- mpr_release_candidates(today, n_months = max_lookback)
  skipped    <- character(0)

  for (rel in candidates) {
    url <- mpr_resolve_url(rel$month, rel$year)
    if (is.null(url)) next

    zip_path <- download_mpr_zip(rel$month, rel$year, cache = cache, url = url)

    if (mpr_zip_is_old_format(zip_path)) {
      if (length(skipped) > 0L) {
        rel_label <- sprintf("%s %d", tools::toTitleCase(rel$month), rel$year)
        cli::cli_warn(c(
          "!" = "Skipping newer MPR release(s) in the Bank of England's new scenario-based format, not parsed by {.fn boe_mpr_forecasts} yet: {.val {skipped}}.",
          "i" = "Returning the most recent compatible release: {rel_label}."
        ))
      }
      return(list(release = rel, zip_path = zip_path))
    }
    skipped <- c(skipped, sprintf("%s %d", tools::toTitleCase(rel$month),
                                  rel$year))
  }

  cli::cli_abort(c(
    "Could not find a compatible MPR release in the last {max_lookback} months.",
    "i" = "Recent releases may use the Bank's new scenario-based format (not parsed by {.fn boe_mpr_forecasts} yet).",
    "i" = "Check your network connection, or request a known release with {.arg month} and {.arg year}."
  ))
}


#' Download the MPR zip with simple time-based caching
#'
#' `url` may be supplied pre-resolved (e.g. by [pick_mpr_release()]) to
#' avoid a second existence probe; otherwise the existing filename
#' variants are probed and the first that resolves is used.
#' @noRd
download_mpr_zip <- function(month, year, cache = TRUE, url = NULL) {
  if (is.null(url)) {
    url <- mpr_resolve_url(month, year)
    if (is.null(url)) {
      cli::cli_abort(c(
        "No MPR data archive found for {.val {month}} {.val {year}}.",
        "i" = "Check the release exists and is published at {.url https://www.bankofengland.co.uk/monetary-policy}."
      ))
    }
  }
  cache_dir  <- boe_cache_dir()
  dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
  cache_file <- file.path(cache_dir, sprintf("mpr-%s-%d.zip", month, year))

  is_latest_year <- year >= as.integer(format(Sys.Date(), "%Y")) - 1L

  if (cache && file.exists(cache_file)) {
    age_h <- as.numeric(difftime(Sys.time(), file.info(cache_file)$mtime,
                                 units = "hours"))
    if (!is_latest_year || age_h < 24) {
      cli::cli_progress_step(sprintf("Using cached %s %d MPR archive",
                                     tools::toTitleCase(month), year))
      return(cache_file)
    }
  }

  cli::cli_progress_step(sprintf("Downloading %s %d MPR archive",
                                 tools::toTitleCase(month), year))

  tryCatch(
    httr2::request(url) |>
      httr2::req_user_agent("boe R package (https://github.com/charlescoverdale/boe)") |>
      httr2::req_timeout(180) |>
      httr2::req_retry(
        max_tries    = 3,
        is_transient = function(resp) httr2::resp_status(resp) %in% c(429L, 503L)
      ) |>
      httr2::req_perform(path = cache_file),
    error = function(e) {
      if (file.exists(cache_file)) unlink(cache_file)
      cli::cli_abort(c(
        "Download failed.",
        "i" = "Check the {.val {month}} {.val {year}} release exists, and your network connection.",
        "x" = conditionMessage(e)
      ))
    }
  )
  cache_file
}


#' Does an MPR zip contain the classic (parseable) Projections Databank?
#'
#' The April 2026 redesign renamed the workbook to "Scenario Projections
#' Databank" and changed its layout. We treat a release as compatible
#' only when it ships a databank workbook whose name does not contain
#' "Scenario".
#' @noRd
mpr_zip_is_old_format <- function(zip_path) {
  files <- tryCatch(utils::unzip(zip_path, list = TRUE)$Name,
                    error = function(e) character(0))
  db <- files[grepl("Projections Databank", files, fixed = TRUE) &
              grepl("\\.xlsx?$", files, ignore.case = TRUE)]
  any(!grepl("Scenario", db, fixed = TRUE))
}


#' Extract the Projections Databank workbook from an MPR zip
#' @noRd
extract_projections_databank <- function(zip_path) {
  files <- utils::unzip(zip_path, list = TRUE)$Name
  hits  <- grep("Projections Databank", files, fixed = TRUE)
  hits  <- hits[grepl("\\.xlsx?$", files[hits], ignore.case = TRUE)]
  # Prefer the classic workbook over the new "Scenario Projections
  # Databank" when both are present.
  classic <- hits[!grepl("Scenario", files[hits], fixed = TRUE)]
  if (length(classic) > 0L) hits <- classic

  if (length(hits) == 0L) {
    cli::cli_abort(c(
      "Projections Databank not found in MPR archive.",
      "i" = "Files in archive: {.val {files}}"
    ))
  }

  out_dir <- tempfile("boe_mpr_")
  dir.create(out_dir, recursive = TRUE)
  utils::unzip(zip_path,
               files     = files[hits[1L]],
               exdir     = out_dir,
               junkpaths = TRUE)
  file.path(out_dir, basename(files[hits[1L]]))
}


#' Parse one projection sheet into long format
#'
#' Sheet layout varies across the Projections Databank: most sheets put
#' the "Date of publication" header at row 5 with Excel serial dates in
#' column 1, but some (e.g. "38. Bank Rate") place it lower and use
#' text publication labels like "November 2004". This parser auto-
#' detects the header row by scanning for quarter labels, then handles
#' both date encodings.
#'
#' @noRd
parse_projection_sheet <- function(path, sheet_name, series_name) {
  sheets <- readxl::excel_sheets(path)
  if (!(sheet_name %in% sheets)) {
    matches <- grep(sub("^\\d+\\.\\s*", "", sheet_name),
                    sheets, ignore.case = TRUE, value = TRUE)
    matches <- matches[!grepl("distribution|short-term", matches, ignore.case = TRUE)]
    if (length(matches) == 0L) {
      cli::cli_abort(c(
        "Sheet {.val {sheet_name}} not found in MPR Projections Databank.",
        "i" = "Available sheets: {.val {head(sheets, 10)}}{if (length(sheets) > 10) ' ...'}"
      ))
    }
    sheet_name <- matches[1L]
  }

  raw <- suppressMessages(readxl::read_excel(
    path, sheet = sheet_name, col_names = FALSE
  ))
  raw <- as.data.frame(raw, stringsAsFactors = FALSE)
  if (nrow(raw) < 6L || ncol(raw) < 2L) {
    cli::cli_abort("Sheet {.val {sheet_name}} is too small to parse.")
  }

  hdr_row <- detect_header_row(raw)
  if (is.na(hdr_row)) {
    cli::cli_abort("Could not detect quarter-label header row in sheet {.val {sheet_name}}.")
  }

  header        <- as.character(unlist(raw[hdr_row, -1L]))
  is_qtr_label  <- !is.na(header) & grepl("^\\d{4}\\s*Q[1-4]$", trimws(header))
  quarter_labels <- trimws(header[is_qtr_label])
  horizon_dates  <- quarter_label_to_date(quarter_labels)

  body      <- raw[(hdr_row + 1L):nrow(raw), c(TRUE, is_qtr_label), drop = FALSE]
  pub_dates <- parse_publication_date(body[[1L]])
  good      <- !is.na(pub_dates)
  body      <- body[good, , drop = FALSE]
  pub_dates <- pub_dates[good]
  if (length(pub_dates) == 0L) {
    cli::cli_abort("No publication-date rows found in sheet {.val {sheet_name}}.")
  }

  vals <- suppressWarnings(matrix(
    as.numeric(as.matrix(body[, -1L, drop = FALSE])),
    nrow = nrow(body),
    ncol = length(quarter_labels)
  ))

  long <- data.frame(
    date         = rep(pub_dates,      times = ncol(vals)),
    horizon      = rep(quarter_labels, each  = nrow(vals)),
    horizon_date = rep(horizon_dates,  each  = nrow(vals)),
    series       = series_name,
    value        = as.numeric(vals),
    stringsAsFactors = FALSE
  )
  long <- long[!is.na(long$value), , drop = FALSE]
  rownames(long) <- NULL
  long
}


#' Find the row of `raw` whose columns contain quarter labels
#'
#' Scans the first 20 rows for a row where 5 or more cells (excluding
#' the first column) match the `YYYY QN` pattern.
#' @noRd
detect_header_row <- function(raw) {
  n_scan <- min(20L, nrow(raw))
  for (r in seq_len(n_scan)) {
    candidates <- as.character(unlist(raw[r, -1L]))
    n_qtrs <- sum(grepl("^\\d{4}\\s*Q[1-4]$", trimws(candidates)),
                  na.rm = TRUE)
    if (n_qtrs >= 5L) return(r)
  }
  NA_integer_
}


#' Parse a publication-date column, accepting Excel serials or text
#'
#' BoE uses Excel serials in most projection sheets but text labels
#' like "November 2004" or "November 2022 (f)" in others. Footnote
#' markers in parentheses are stripped before parsing the text form.
#' Text dates resolve to the first day of the named month.
#' @noRd
parse_publication_date <- function(x) {
  serials <- suppressWarnings(as.numeric(x))
  out     <- rep(as.Date(NA), length(x))
  ok_ser  <- !is.na(serials) & serials > 30000 & serials < 60000
  out[ok_ser] <- as.Date(serials[ok_ser], origin = "1899-12-30")

  todo <- !ok_ser & !is.na(x) & nzchar(as.character(x))
  if (any(todo)) {
    txt <- sub("\\s*\\([a-z]\\)\\s*$", "", as.character(x[todo]),
               ignore.case = TRUE)
    old <- Sys.getlocale("LC_TIME")
    on.exit(suppressWarnings(Sys.setlocale("LC_TIME", old)), add = TRUE)
    suppressWarnings(Sys.setlocale("LC_TIME", "C"))
    out[todo] <- suppressWarnings(
      as.Date(paste("1", txt), format = "%d %B %Y")
    )
  }
  out
}


#' Convert "2024 Q1" to a Date at the start of the quarter
#' @noRd
quarter_label_to_date <- function(labels) {
  parts <- strsplit(trimws(labels), "\\s+")
  vapply(parts, function(p) {
    if (length(p) < 2L) return(NA_character_)
    yr <- as.integer(p[1L])
    qn <- as.integer(sub("Q", "", p[2L]))
    if (is.na(yr) || is.na(qn) || qn < 1L || qn > 4L) return(NA_character_)
    sprintf("%d-%02d-01", yr, (qn - 1L) * 3L + 1L)
  }, character(1L)) |> as.Date()
}

# Tests for boe_mpr_forecasts() (parses BoE Projections Databank).

test_that("mpr_zip_url / mpr_zip_urls handle the filename drift", {
  # Primary best guess: singular "chart" for <= 2024, plural for >= 2025.
  expect_match(mpr_zip_url("november", 2024L), "chart-slides-and-data\\.zip$")
  expect_match(mpr_zip_url("february", 2026L), "charts-slides-and-data\\.zip$")
  expect_match(mpr_zip_url("november", 2024L),
               "/monetary-policy-report/2024/november/")

  # Both filename variants are offered so the caller can probe each.
  urls <- mpr_zip_urls("may", 2025L)
  expect_length(urls, 2L)
  expect_true(any(grepl("charts-slides-and-data.zip", urls, fixed = TRUE)))
  expect_true(any(grepl("mpr-may-2025-chart-slides-and-data.zip", urls,
                        fixed = TRUE)))
})

test_that("mpr_release_candidates lists recent months, newest first", {
  cands <- mpr_release_candidates(today = as.Date("2026-05-29"), n_months = 4L)
  expect_length(cands, 4L)
  expect_equal(cands[[1L]], list(month = "may",      year = 2026L))
  expect_equal(cands[[2L]], list(month = "april",    year = 2026L))
  expect_equal(cands[[4L]], list(month = "february", year = 2026L))

  # Year rollover when walking back past January.
  roll <- mpr_release_candidates(today = as.Date("2026-01-15"), n_months = 3L)
  expect_equal(roll[[1L]], list(month = "january",  year = 2026L))
  expect_equal(roll[[2L]], list(month = "december", year = 2025L))
  expect_equal(roll[[3L]], list(month = "november", year = 2025L))
})

test_that("resolve_mpr_release validates inputs", {
  # Any real month name is accepted: the schedule drifts (the 2026 Q2
  # report was published in April, not May), so we no longer hard-code
  # a Feb/May/Aug/Nov calendar.
  expect_equal(resolve_mpr_release(month = "april", year = 2026L),
               list(month = "april", year = 2026L))
  expect_error(resolve_mpr_release(month = "smarch", year = 2026L), "month name")
  expect_error(resolve_mpr_release(month = "february", year = 2018L), "2019")
  expect_error(resolve_mpr_release(month = "february", year = NULL), "both")
})

test_that("quarter_label_to_date parses the BoE format", {
  d <- quarter_label_to_date(c("2024 Q1", "2024 Q4", "2025 Q2"))
  expect_equal(d, as.Date(c("2024-01-01", "2024-10-01", "2025-04-01")))
})

test_that("URL probing survives schedule drift and filename variants", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()

  # Regression: the 2026 Q2 report was published in April, not May. The
  # old code guessed May and 404'd; the resolver must reflect reality.
  expect_null(mpr_resolve_url("may", 2026L))
  expect_false(is.null(mpr_resolve_url("april", 2026L)))

  # The data archive filename drifted from "chart-slides-and-data"
  # (February 2025, singular) to "charts-slides-and-data" (August 2025,
  # plural). Both must resolve via the variant probing.
  expect_false(is.null(mpr_resolve_url("february", 2025L)))
  expect_false(is.null(mpr_resolve_url("august", 2025L)))

  # The existence check distinguishes a live archive from a 404.
  expect_true(url_exists_boe(mpr_zip_url("august", 2025L)))
  expect_false(url_exists_boe(mpr_zip_url("may", 2026L)))
})

test_that("pick_mpr_release returns a real, parseable release", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()

  op <- options(boe.cache_dir = tempfile("boe_pick_"))
  on.exit(options(op), add = TRUE)

  picked <- suppressWarnings(pick_mpr_release())

  # The crux of the original bug: the auto-selected release must
  # actually be downloadable (no 404) and parseable (classic format).
  expect_false(is.null(mpr_resolve_url(picked$release$month,
                                       picked$release$year)))
  expect_true(file.exists(picked$zip_path))
  expect_true(mpr_zip_is_old_format(picked$zip_path))
})

test_that("boe_mpr_forecasts fetches and parses the latest compatible release", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("readxl")

  op <- options(boe.cache_dir = tempfile("boe_mpr_"))
  on.exit(options(op), add = TRUE)

  # suppressWarnings: when the latest release uses the unsupported
  # scenario format, the function warns and falls back to an earlier one.
  out <- suppressWarnings(
    boe_mpr_forecasts(series = c("cpi_inflation", "bank_rate"))
  )

  expect_s3_class(out, "boe_tbl")
  expected_cols <- c("date", "horizon", "horizon_date", "series", "value")
  expect_true(all(expected_cols %in% names(out)))
  expect_gt(nrow(out), 100L)
  expect_true(all(out$series %in% c("cpi_inflation", "bank_rate")))
  expect_true(all(grepl("^\\d{4} Q[1-4]$", out$horizon)))
  expect_s3_class(out$horizon_date, "Date")

  q <- attr(out, "boe_query")
  expect_equal(q$function_name, "boe_mpr_forecasts")
})

# utils.R — internal helpers for BoE API access + caching

BOE_API <- "https://www.bankofengland.co.uk/boeapps/database/_iadb-fromshowcolumns.asp"

#' Fetch series from the Bank of England Interactive Statistical Database
#'
#' @param series_codes Character vector of BoE series codes.
#' @param from Date or character. Start date (inclusive).
#' @param to Date or character. End date (inclusive). Defaults to today.
#' @param cache Logical. Use cached response if available?
#' @return A data frame with columns: date, code, value.
#' @noRd
boe_fetch <- function(series_codes, from, to = Sys.Date(), cache = TRUE) {
  from <- as.Date(from)
  to   <- as.Date(to)
  if (from > to) {
    cli::cli_abort("{.arg from} ({.val {from}}) must be before {.arg to} ({.val {to}}).")
  }

  from_str <- format_boe_date(from)
  to_str   <- format_boe_date(to)
  codes    <- paste(series_codes, collapse = ",")

  url <- paste0(
    BOE_API,
    "?csv.x=yes",
    "&Datefrom=", from_str,
    "&Dateto=", to_str,
    "&SeriesCodes=", codes,
    "&UsingCodes=Y",
    "&CSVF=TN"
  )

  path <- download_cached_boe(url, cache = cache)
  parse_boe_csv(path, series_codes)
}

#' Format a date for the BoE API (DD/Mon/YYYY)
#' @noRd
format_boe_date <- function(x) {
  if (is.character(x)) {
    x <- as.Date(x)
  }
  format(x, "%d/%b/%Y")
}

#' Parse BoE CSV response into a tidy data frame
#' @noRd
parse_boe_csv <- function(path, expected_codes) {
  lines <- readLines(path, warn = FALSE)

  # BoE returns an HTML error page for invalid series

  if (length(lines) == 0 || any(grepl("<title>Error</title>", lines, ignore.case = TRUE))) {
    cli::cli_abort(c(
      "The Bank of England returned an error.",
      "i" = "Check that your series codes are valid: {.val {expected_codes}}"
    ))
  }

  # Filter out blank lines
  lines <- lines[nzchar(trimws(lines))]
  if (length(lines) < 2) {
    cli::cli_abort("No data returned for series: {.val {expected_codes}}")
  }

  raw <- utils::read.csv(text = paste(lines, collapse = "\n"),
                          stringsAsFactors = FALSE, check.names = FALSE)

  if (nrow(raw) == 0) {
    cli::cli_abort("No data returned for series: {.val {expected_codes}}")
  }

  # First column is DATE; remaining columns are series codes
  date_col <- raw[[1]]
  dates    <- as.Date(trimws(date_col), format = "%d %b %Y")

  code_cols <- names(raw)[-1]
  results <- list()

  for (col in code_cols) {
    vals <- suppressWarnings(as.numeric(raw[[col]]))
    results[[length(results) + 1]] <- data.frame(
      date  = dates,
      code  = col,
      value = vals,
      stringsAsFactors = FALSE
    )
  }

  out <- do.call(rbind, results)
  # Drop rows with NA dates or values (from mixed-frequency gaps)
  out <- out[!is.na(out$date) & !is.na(out$value), ]
  rownames(out) <- NULL
  out
}

#' Get the cache directory, respecting the boe.cache_dir option
#' @noRd
boe_cache_dir <- function() {
  getOption("boe.cache_dir", default = tools::R_user_dir("boe", "cache"))
}

#' Download a URL with local caching
#' @noRd
download_cached_boe <- function(url, cache = TRUE) {
  cache_dir  <- boe_cache_dir()
  ext        <- ".csv"
  cache_file <- file.path(cache_dir, paste0(digest_url_boe(url), ext))

  if (cache && file.exists(cache_file)) {
    cli::cli_progress_step("Using cached data")
    return(cache_file)
  }

  dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
  cli::cli_progress_step("Downloading from Bank of England")

  tryCatch(
    httr2::request(url) |>
      httr2::req_user_agent("boe R package (https://github.com/charlescoverdale/boe)") |>
      httr2::req_throttle(rate = 5 / 10) |>
      httr2::req_retry(
        max_tries    = 3,
        is_transient = function(resp) httr2::resp_status(resp) %in% c(429L, 503L)
      ) |>
      httr2::req_timeout(60) |>
      httr2::req_perform(path = cache_file),
    error = function(e) {
      if (file.exists(cache_file)) unlink(cache_file)
      cli::cli_abort(c(
        "Download failed.",
        "i" = "Check your internet connection.",
        "x" = conditionMessage(e)
      ))
    }
  )

  cache_file
}

#' Simple URL hash for cache filenames
#' @noRd
digest_url_boe <- function(url) {
  chars    <- utf8ToInt(url)
  weights  <- seq_along(chars)
  checksum <- sum(as.numeric(chars) * weights) %% (2^31 - 1)
  sprintf("%010.0f_%04d", as.numeric(checksum), nchar(url) %% 10000L)
}

#' Validate and parse date arguments
#' @noRd
parse_date_arg <- function(x, arg_name = "from") {
  if (is.null(x)) return(NULL)
  if (inherits(x, "Date")) return(x)
  d <- tryCatch(as.Date(x), error = function(e) NA)
  if (is.na(d)) {
    cli::cli_abort("{.arg {arg_name}} must be a valid date (e.g. {.val 2020-01-01}).")
  }
  d
}

# S3 class for Bank of England query results.

#' Construct a boe_tbl
#'
#' Internal constructor for the `boe_tbl` S3 class. Wraps a data frame
#' with provenance metadata so downstream users can trace any returned
#' table back to the BoE series, date range, and request that produced
#' it.
#'
#' @param df A data frame.
#' @param query A list of query metadata. Recognised fields include
#'   `series_codes`, `from`, `to`, `frequency`, `function_name`,
#'   `source_url`, `fetched_at`, `vintage`.
#' @return A `boe_tbl`, which is a subclass of `data.frame`.
#' @noRd
new_boe_tbl <- function(df, query = list()) {
  if (!is.data.frame(df)) {
    df <- as.data.frame(df, stringsAsFactors = FALSE)
  }
  if (is.null(query$fetched_at)) {
    query$fetched_at <- Sys.time()
  }
  attr(df, "boe_query") <- query
  class(df) <- c("boe_tbl", "data.frame")
  df
}


#' Print method for boe_tbl
#'
#' Adds a one-line provenance header above the data frame body. The
#' header summarises the request: number of series (and codes if few),
#' observation count, date range, frequency, and any vintage tag.
#'
#' @param x A `boe_tbl`.
#' @param ... Passed to the underlying `print.data.frame` method.
#' @return `x`, invisibly.
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' x <- boe_bank_rate(from = "2020-01-01", frequency = "monthly")
#' print(x)
#' options(op)
#' }
#' @export
print.boe_tbl <- function(x, ...) {
  q <- attr(x, "boe_query")
  parts <- character(0L)

  if (!is.null(q$series_codes)) {
    codes <- unname(q$series_codes)
    n <- length(codes)
    label <- sprintf("%d series", n)
    if (n >= 1L && n <= 5L) {
      label <- paste0(label, " [", paste(codes, collapse = ","), "]")
    }
    parts <- c(parts, label)
  }

  parts <- c(parts, sprintf("%d obs", nrow(x)))

  if (!is.null(q$from) && !is.null(q$to)) {
    parts <- c(parts, sprintf("%s to %s",
                              format(as.Date(q$from)),
                              format(as.Date(q$to))))
  }
  if (!is.null(q$frequency) && nzchar(q$frequency)) {
    parts <- c(parts, paste0("freq=", q$frequency))
  }
  if (!is.null(q$vintage) && nzchar(q$vintage)) {
    parts <- c(parts, paste0("as_of=", q$vintage))
  }

  header <- if (!is.null(q$function_name) && nzchar(q$function_name)) {
    sprintf("# BoE [%s]: %s", q$function_name,
            paste(parts, collapse = " \u00b7 "))
  } else {
    sprintf("# BoE: %s", paste(parts, collapse = " \u00b7 "))
  }
  cat(header, "\n", sep = "")
  NextMethod()
}

# Tests for the boe_tbl S3 class.

test_that("new_boe_tbl returns a data.frame subclass", {
  df <- data.frame(date = Sys.Date(), value = 1)
  x  <- new_boe_tbl(df, query = list(series_codes = "IUDBEDR"))

  expect_s3_class(x, "boe_tbl")
  expect_s3_class(x, "data.frame")
  expect_true(is.data.frame(x))
})

test_that("new_boe_tbl attaches query metadata", {
  df <- data.frame(date = Sys.Date(), value = 1)
  q  <- list(series_codes = c("IUDBEDR", "IUDSOIA"),
             from = as.Date("2020-01-01"),
             to   = as.Date("2020-12-31"),
             frequency = "daily",
             function_name = "boe_get")

  x   <- new_boe_tbl(df, query = q)
  got <- attr(x, "boe_query")

  expect_equal(got$series_codes, c("IUDBEDR", "IUDSOIA"))
  expect_equal(got$frequency, "daily")
  expect_equal(got$function_name, "boe_get")
  expect_s3_class(got$fetched_at, "POSIXct")
})

test_that("new_boe_tbl coerces non-data-frames", {
  m <- matrix(1:4, ncol = 2, dimnames = list(NULL, c("a", "b")))
  x <- new_boe_tbl(m)
  expect_s3_class(x, "boe_tbl")
  expect_true(is.data.frame(x))
})

test_that("new_boe_tbl preserves a user-supplied fetched_at", {
  ts <- as.POSIXct("2026-01-01 00:00:00", tz = "UTC")
  x  <- new_boe_tbl(data.frame(a = 1), query = list(fetched_at = ts))
  expect_equal(attr(x, "boe_query")$fetched_at, ts)
})

test_that("print.boe_tbl emits a provenance header", {
  df <- data.frame(date = as.Date("2020-01-01") + 0:9, value = 1:10)
  x  <- new_boe_tbl(df, query = list(
    series_codes  = "IUDBEDR",
    from          = as.Date("2020-01-01"),
    to            = as.Date("2020-01-10"),
    frequency     = "daily",
    function_name = "boe_bank_rate"
  ))

  out <- utils::capture.output(print(x))

  expect_true(any(grepl("^# BoE \\[boe_bank_rate\\]:", out)))
  expect_true(any(grepl("1 series \\[IUDBEDR\\]", out)))
  expect_true(any(grepl("10 obs", out)))
  expect_true(any(grepl("freq=daily", out)))
  expect_true(any(grepl("2020-01-01 to 2020-01-10", out)))
})

test_that("print.boe_tbl falls back when function_name is missing", {
  df <- data.frame(a = 1)
  x  <- new_boe_tbl(df, query = list(series_codes = "IUDBEDR"))
  out <- utils::capture.output(print(x))
  expect_true(any(grepl("^# BoE: ", out)))
})

test_that("print.boe_tbl summarises many-series queries without listing all codes", {
  codes <- sprintf("XCODE%02d", 1:10)
  df <- data.frame(a = 1)
  x  <- new_boe_tbl(df, query = list(series_codes = codes))
  out <- utils::capture.output(print(x))
  expect_true(any(grepl("10 series", out)))
  expect_false(any(grepl("XCODE01,XCODE02", out)))
})

#' Clear locally cached Bank of England data
#'
#' Removes cached data files downloaded from the Bank of England.
#'
#' @param max_age_days Numeric or `NULL`. If specified, only removes files
#'   older than this many days. If `NULL` (the default), removes all
#'   cached files.
#'
#' @return Invisibly returns the number of files removed.
#'
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' # Remove files older than 7 days
#' clear_cache(max_age_days = 7)
#'
#' # Remove everything
#' clear_cache()
#' options(op)
#' }
#'
#' @family data access
#' @export
clear_cache <- function(max_age_days = NULL) {
  cache_dir <- boe_cache_dir()

  if (!dir.exists(cache_dir)) {
    cli::cli_alert_info("No cache directory found.")
    return(invisible(0L))
  }

  files <- list.files(cache_dir, full.names = TRUE)

  if (length(files) == 0) {
    cli::cli_alert_info("Cache is empty.")
    return(invisible(0L))
  }

  if (!is.null(max_age_days)) {
    info    <- file.info(files)
    age     <- difftime(Sys.time(), info$mtime, units = "days")
    files   <- files[age > max_age_days]
  }

  if (length(files) == 0) {
    cli::cli_alert_info("No files older than {max_age_days} day{?s}.")
    return(invisible(0L))
  }

  unlink(files)
  cli::cli_alert_success("Removed {length(files)} cached file{?s}.")
  invisible(length(files))
}

#' Download consumer credit outstanding
#'
#' Downloads monthly outstanding amounts of consumer credit (total,
#' credit cards, and other consumer credit). Seasonally adjusted.
#' Available from April 1993.
#'
#' @param type Character vector. One or more of `"total"`, `"credit_card"`,
#'   `"other"`. Defaults to all three.
#' @param from Date or character (YYYY-MM-DD). Start date. Defaults to
#'   `"1993-04-01"`.
#' @param to Date or character (YYYY-MM-DD). End date. Defaults to today.
#' @param cache Logical. Use cached data if available (default `TRUE`).
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{date}{Date. End of month.}
#'     \item{type}{Character. Credit type.}
#'     \item{amount_gbp_m}{Numeric. Outstanding amount (millions of
#'       pounds).}
#'   }
#'
#' @source
#' <https://www.bankofengland.co.uk/boeapps/database/>
#'
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' boe_consumer_credit(from = "2015-01-01")
#' options(op)
#' }
#'
#' @family credit and housing
#' @export
boe_consumer_credit <- function(type  = c("total", "credit_card", "other"),
                                from  = "1993-04-01",
                                to    = Sys.Date(),
                                cache = TRUE) {

  type <- match.arg(type, several.ok = TRUE)

  code_map <- c(
    "total"       = "LPMVZRI",
    "credit_card" = "LPMVZRJ",
    "other"       = "LPMVZRK"
  )

  codes <- code_map[type]

  from <- parse_date_arg(from, "from")
  to   <- parse_date_arg(to, "to")

  out <- boe_fetch(codes, from = from, to = to, cache = cache)

  code_to_type <- stats::setNames(names(codes), codes)
  out$type <- code_to_type[out$code]

  result <- data.frame(
    date        = out$date,
    type        = out$type,
    amount_gbp_m = out$value,
    stringsAsFactors = FALSE
  )

  result <- result[order(result$type, result$date), ]
  rownames(result) <- NULL

  cli::cli_progress_done()
  new_boe_tbl(result, query = list(
    series_codes  = unname(codes),
    from          = from,
    to            = to,
    frequency     = "monthly",
    function_name = "boe_consumer_credit"
  ))
}

test_that("boe_consumer_credit() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_consumer_credit(from = "2024-01-01", to = "2024-06-30")

  expect_s3_class(out, "data.frame")
  expect_named(out, c("date", "type", "amount_gbp_m"))
  expect_true(nrow(out) > 0)
  expect_true(all(out$type %in% c("total", "credit_card", "other")))
})

test_that("boe_consumer_credit() type filter works", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_consumer_credit(type = "credit_card", from = "2024-01-01", to = "2024-06-30")
  expect_true(all(out$type == "credit_card"))
})

#' Wide panel of BoE yield curve at chosen pillar maturities
#'
#' Convenience wrapper around `boe_curve()` that returns a wide-format
#' panel: one row per date, one column per requested pillar maturity.
#' This is the form most users want for time-series modelling and quick
#' plotting.
#'
#' For each requested pillar, the function picks the published maturity
#' closest to the request (within a 0.05-year tolerance) and uses that.
#' The standard grid steps in 0.5-year increments and the short-end grid
#' (`segment = "short"`) in monthly increments, so pillars at integer,
#' half-integer, or whole-month maturities align exactly.
#'
#' @inheritParams boe_curve
#' @param maturities Numeric vector of pillar maturities in years. When
#'   `NULL` (default) a sensible set is chosen for the segment:
#'   `c(0.5, 1, 2, 5, 10, 20)` for `"standard"` and
#'   `c(0.5, 1, 2, 3, 5)` for `"short"`. Pillars not on the published grid
#'   for the chosen curve and segment are dropped with a warning.
#'
#' @return A `boe_tbl` data frame with columns `date` and one numeric
#'   column per pillar named like `m0.5`, `m1`, `m2`, `m5`, `m10`, `m20`.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("readxl", quietly = TRUE)) {
#'   op <- options(boe.cache_dir = tempdir())
#'   # Latest month: wide panel at chosen pillar maturities
#'   panel <- boe_curve_panel(curve = "nominal", measure = "spot",
#'                            maturities = c(2, 5, 10, 20))
#'   head(panel)
#'   options(op)
#' }
#' }
#'
#' \dontrun{
#' # Historical panel (multi-decade archive download; not run automatically)
#' hist <- boe_curve_panel(curve = "nominal", measure = "spot",
#'                         from = "2020-01-01", maturities = c(2, 5, 10, 20))
#' }
#'
#' @family interest rates
#' @export
boe_curve_panel <- function(curve       = c("nominal", "real", "inflation", "ois", "blc"),
                            measure     = c("spot", "forward"),
                            segment     = c("standard", "short"),
                            frequency   = c("daily", "monthly"),
                            from        = NULL,
                            to          = NULL,
                            maturities  = NULL,
                            cache       = TRUE,
                            cache_ttl_h = NULL) {

  curve     <- match.arg(curve)
  measure   <- match.arg(measure)
  segment   <- match.arg(segment)
  frequency <- match.arg(frequency)

  if (is.null(maturities)) {
    maturities <- if (segment == "short") c(0.5, 1, 2, 3, 5)
                  else                     c(0.5, 1, 2, 5, 10, 20)
  }
  if (!is.numeric(maturities) || any(maturities <= 0)) {
    cli::cli_abort("{.arg maturities} must be a positive numeric vector.")
  }
  maturities <- sort(unique(maturities))

  long <- boe_curve(
    curve       = curve,
    measure     = measure,
    segment     = segment,
    frequency   = frequency,
    from        = from,
    to          = to,
    cache       = cache,
    cache_ttl_h = cache_ttl_h
  )

  if (nrow(long) == 0L) {
    out <- data.frame(date = as.Date(character()))
    for (m in maturities) out[[sprintf("m%g", m)]] <- numeric()
    return(new_boe_tbl(out, query = c(attr(long, "boe_query"),
                                      list(maturities = maturities))))
  }

  available <- sort(unique(long$maturity_years))
  matched <- vapply(maturities, function(m) {
    diffs <- abs(available - m)
    if (min(diffs) > 0.05) return(NA_real_)
    available[which.min(diffs)]
  }, numeric(1))

  # Drop pillars off the published grid, but keep the requested pillar
  # labels aligned with the maturities they matched (filtering both with
  # the same mask) so surviving columns are never mislabelled.
  keep        <- !is.na(matched)
  if (any(!keep)) {
    dropped <- maturities[!keep]
    cli::cli_warn(c(
      "{length(dropped)} requested pillar{?s} not on the {curve} {segment} grid; dropped: {.val {dropped}}.",
      "i" = "Available maturities span {.val {min(available)}} to {.val {max(available)}} years."
    ))
  }
  pillars     <- maturities[keep]
  matched_mat <- matched[keep]
  if (length(matched_mat) == 0L) {
    cli::cli_abort("None of the requested pillars matched the published grid.")
  }

  dates <- sort(unique(long$date))
  out <- data.frame(date = dates, stringsAsFactors = FALSE)

  for (i in seq_along(matched_mat)) {
    m <- matched_mat[[i]]
    label <- sprintf("m%g", pillars[[i]])
    sub <- long[abs(long$maturity_years - m) < 1e-8, c("date", "rate_pct"),
                drop = FALSE]
    out[[label]] <- sub$rate_pct[match(out$date, sub$date)]
  }

  q <- attr(long, "boe_query")
  q$maturities    <- maturities
  q$function_name <- "boe_curve_panel"

  new_boe_tbl(out, query = q)
}

#' Bank Rate decision history
#'
#' Returns the history of Monetary Policy Committee decisions to change
#' Bank Rate, derived from the daily Bank Rate series. Each row is one
#' rate-change event, showing the effective date, the new rate, the
#' previous rate, and the change in basis points. Holds (meetings where
#' the rate was unchanged) are not included; for the full meeting-level
#' record including holds, see [boe_mpc_votes()].
#'
#' @param from Date or character. Start date. Defaults to
#'   `"1997-06-06"` (the first MPC meeting).
#' @param to Date or character. End date. Defaults to today.
#' @param cache Logical. Use cached Bank Rate data if available
#'   (default `TRUE`).
#'
#' @return A `boe_tbl` data frame with columns:
#'   \describe{
#'     \item{date}{Date. Effective date of the rate change.}
#'     \item{new_rate_pct}{Numeric. Bank Rate after the decision (percent).}
#'     \item{prev_rate_pct}{Numeric. Bank Rate before the decision (percent).}
#'     \item{change_bps}{Integer. Change in basis points (positive = hike,
#'       negative = cut).}
#'     \item{direction}{Character. `"hike"` or `"cut"`.}
#'   }
#'
#' @source
#' Derived from BoE series `IUDBEDR` (daily Bank Rate). See
#' <https://www.bankofengland.co.uk/monetary-policy>.
#'
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' # All MPC decisions since the global financial crisis
#' boe_mpc_decisions(from = "2007-01-01")
#'
#' # Just decisions in 2024 to date
#' boe_mpc_decisions(from = "2024-01-01")
#' options(op)
#' }
#'
#' @family monetary policy
#' @seealso [boe_bank_rate()], [boe_mpc_votes()]
#' @export
boe_mpc_decisions <- function(from  = "1997-06-06",
                              to    = Sys.Date(),
                              cache = TRUE) {

  br <- boe_bank_rate(from = from, to = to,
                      frequency = "daily", cache = cache)

  rates <- br$rate_pct
  dates <- br$date
  if (length(rates) < 2L) {
    cli::cli_abort("Not enough Bank Rate observations to detect changes.")
  }

  dr      <- diff(rates)
  changes <- which(dr != 0)
  if (length(changes) == 0L) {
    out <- data.frame(
      date          = as.Date(character(0)),
      new_rate_pct  = numeric(0),
      prev_rate_pct = numeric(0),
      change_bps    = integer(0),
      direction     = character(0),
      stringsAsFactors = FALSE
    )
  } else {
    out <- data.frame(
      date          = dates[changes + 1L],
      new_rate_pct  = rates[changes + 1L],
      prev_rate_pct = rates[changes],
      change_bps    = as.integer(round((rates[changes + 1L] - rates[changes]) * 100)),
      direction     = ifelse(dr[changes] > 0, "hike", "cut"),
      stringsAsFactors = FALSE
    )
  }

  new_boe_tbl(out, query = list(
    series_codes  = "IUDBEDR",
    from          = from,
    to            = to,
    frequency     = "decision",
    function_name = "boe_mpc_decisions"
  ))
}

# Tests for boe_mpc_decisions().

test_that("boe_mpc_decisions returns the expected schema", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()

  op <- options(boe.cache_dir = tempfile("boe_mpc_dec_"))
  on.exit(options(op), add = TRUE)

  out <- boe_mpc_decisions(from = "2020-01-01", to = "2024-12-31")

  expect_s3_class(out, "boe_tbl")
  expected_cols <- c("date", "new_rate_pct", "prev_rate_pct",
                     "change_bps", "direction")
  expect_true(all(expected_cols %in% names(out)))
  expect_true(all(out$direction %in% c("hike", "cut")))
  expect_true(all(is.finite(out$change_bps)))
  expect_true(all(out$change_bps != 0L))
  expect_equal(out$new_rate_pct - out$prev_rate_pct,
               out$change_bps / 100, tolerance = 1e-6)

  q <- attr(out, "boe_query")
  expect_equal(q$function_name, "boe_mpc_decisions")
})

test_that("boe_mpc_decisions captures known events (March 2020 cut)", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()

  op <- options(boe.cache_dir = tempfile("boe_mpc_known_"))
  on.exit(options(op), add = TRUE)

  out <- boe_mpc_decisions(from = "2020-03-01", to = "2020-03-31")
  expect_gt(nrow(out), 0L)
  expect_true(all(out$direction == "cut"))
})

#' Download quoted mortgage interest rates
#'
#' Downloads monthly quoted (advertised) mortgage rates from the Bank of
#' England, including fixed-rate products and the standard variable rate
#' (SVR). Available from January 1995.
#'
#' @param type Character vector. One or more of `"2yr_fixed"`,
#'   `"3yr_fixed"`, `"5yr_fixed"`, `"svr"`. Defaults to all four.
#' @param from Date or character (YYYY-MM-DD). Start date. Defaults to
#'   `"1995-01-01"`.
#' @param to Date or character (YYYY-MM-DD). End date. Defaults to today.
#' @param cache Logical. Use cached data if available (default `TRUE`).
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{date}{Date. End of month.}
#'     \item{type}{Character. Mortgage product type.}
#'     \item{rate_pct}{Numeric. Quoted rate (percent).}
#'   }
#'
#' @source
#' <https://www.bankofengland.co.uk/boeapps/database/>
#'
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' # All mortgage rate types since 2015
#' boe_mortgage_rates(from = "2015-01-01")
#'
#' # 2-year fixed only
#' boe_mortgage_rates(type = "2yr_fixed", from = "2020-01-01")
#' options(op)
#' }
#'
#' @family credit and housing
#' @export
boe_mortgage_rates <- function(type  = c("2yr_fixed", "3yr_fixed", "5yr_fixed", "svr"),
                               from  = "1995-01-01",
                               to    = Sys.Date(),
                               cache = TRUE) {

  type <- match.arg(type, several.ok = TRUE)

  code_map <- c(
    "2yr_fixed" = "IUMBV34",
    "3yr_fixed" = "IUMBV37",
    "5yr_fixed" = "IUMBV42",
    "svr"       = "IUMTLMV"
  )

  codes <- code_map[type]

  from <- parse_date_arg(from, "from")
  to   <- parse_date_arg(to, "to")

  out <- boe_fetch(codes, from = from, to = to, cache = cache)

  code_to_type <- stats::setNames(names(codes), codes)
  out$type <- code_to_type[out$code]

  result <- data.frame(
    date     = out$date,
    type     = out$type,
    rate_pct = out$value,
    stringsAsFactors = FALSE
  )

  result <- result[order(result$type, result$date), ]
  rownames(result) <- NULL

  cli::cli_progress_done()
  new_boe_tbl(result, query = list(
    series_codes  = unname(codes),
    from          = from,
    to            = to,
    frequency     = "monthly",
    function_name = "boe_mortgage_rates"
  ))
}

test_that("boe_mortgage_rates() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_mortgage_rates(from = "2024-01-01", to = "2024-06-30")

  expect_s3_class(out, "data.frame")
  expect_named(out, c("date", "type", "rate_pct"))
  expect_true(nrow(out) > 0)
  expect_true(all(out$type %in% c("2yr_fixed", "3yr_fixed", "5yr_fixed", "svr")))
})

test_that("boe_mortgage_rates() type filter works", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_mortgage_rates(type = "2yr_fixed", from = "2024-01-01", to = "2024-06-30")
  expect_true(all(out$type == "2yr_fixed"))
})

#' Download M4 money supply
#'
#' Downloads monthly M4 (broad money) amounts outstanding from the Bank
#' of England. Available from June 1982.
#'
#' @param from Date or character (YYYY-MM-DD). Start date. Defaults to
#'   `"1982-06-01"`.
#' @param to Date or character (YYYY-MM-DD). End date. Defaults to today.
#' @param seasonally_adjusted Logical. Return seasonally adjusted series
#'   (default `TRUE`) or non-seasonally adjusted (`FALSE`).
#' @param cache Logical. Use cached data if available (default `TRUE`).
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{date}{Date. End of month.}
#'     \item{amount_gbp_m}{Numeric. M4 amounts outstanding (millions of
#'       pounds).}
#'   }
#'
#' @source
#' <https://www.bankofengland.co.uk/boeapps/database/>
#'
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' boe_money_supply(from = "2000-01-01")
#' options(op)
#' }
#'
#' @family money
#' @export
boe_money_supply <- function(from                = "1982-06-01",
                             to                  = Sys.Date(),
                             seasonally_adjusted = TRUE,
                             cache               = TRUE) {

  code <- if (seasonally_adjusted) "LPMAUYN" else "LPMAUYM"

  from <- parse_date_arg(from, "from")
  to   <- parse_date_arg(to, "to")

  out <- boe_fetch(code, from = from, to = to, cache = cache)

  result <- data.frame(
    date         = out$date,
    amount_gbp_m = out$value,
    stringsAsFactors = FALSE
  )

  cli::cli_progress_done()
  new_boe_tbl(result, query = list(
    series_codes  = code,
    from          = from,
    to            = to,
    frequency     = "monthly",
    function_name = "boe_money_supply"
  ))
}

test_that("boe_money_supply() returns expected structure", {
  skip_on_cran()
  skip_if_offline()

  out <- boe_money_supply(from = "2024-01-01", to = "2024-06-30")

  expect_s3_class(out, "data.frame")
  expect_named(out, c("date", "amount_gbp_m"))
  expect_s3_class(out$date, "Date")
  expect_type(out$amount_gbp_m, "double")
  expect_true(nrow(out) > 0)
})

#' Download Bank of England Bank Rate history
#'
#' Downloads the official Bank Rate (base interest rate) set by the Monetary
#' Policy Committee. Available as a daily series from January 1975.
#'
#' @param from Date or character (YYYY-MM-DD). Start date. Defaults to
#'   `"1975-01-02"`.
#' @param to Date or character (YYYY-MM-DD). End date. Defaults to today.
#' @param frequency Character. One of `"daily"` (default) or `"monthly"`
#'   (monthly average).
#' @param cache Logical. Use cached data if available (default `TRUE`).
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{date}{Date. Observation date.}
#'     \item{rate_pct}{Numeric. Bank Rate (percent).}
#'   }
#'
#' @source
#' <https://www.bankofengland.co.uk/boeapps/database/>
#'
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' # Bank Rate since 2000
#' boe_bank_rate(from = "2000-01-01")
#'
#' # Monthly average
#' boe_bank_rate(from = "2020-01-01", frequency = "monthly")
#' options(op)
#' }
#'
#' @family interest rates
#' @export
boe_bank_rate <- function(from      = "1975-01-02",
                          to        = Sys.Date(),
                          frequency = c("daily", "monthly"),
                          cache     = TRUE) {

  frequency <- match.arg(frequency)
  code <- if (frequency == "daily") "IUDBEDR" else "IUMABEDR"

  from <- parse_date_arg(from, "from")
  to   <- parse_date_arg(to, "to")

  out <- boe_fetch(code, from = from, to = to, cache = cache)

  result <- data.frame(
    date     = out$date,
    rate_pct = out$value,
    stringsAsFactors = FALSE
  )

  cli::cli_progress_done()
  new_boe_tbl(result, query = list(
    series_codes  = code,
    from          = from,
    to            = to,
    frequency     = frequency,
    function_name = "boe_bank_rate"
  ))
}

#' Catalogue of BoE series wrapped by this package
#'
#' A reference data frame of Bank of England Statistical Database series
#' codes for which the package provides a named convenience function.
#' Used by [boe_search()] and [boe_browse()].
#'
#' @format A data frame with 8 columns:
#' \describe{
#'   \item{code}{Character. BoE series code (e.g. `"IUDBEDR"`).}
#'   \item{title}{Character. Human-readable description.}
#'   \item{category}{Character. Topic grouping. One of `"interest_rates"`,
#'     `"exchange_rates"`, `"mortgage_market"`, `"consumer_credit"`,
#'     `"monetary_aggregates"`.}
#'   \item{frequency}{Character. Native publication frequency
#'     (`"daily"`, `"monthly"`, `"annual"`).}
#'   \item{unit}{Character. Unit of measurement (`"percent"`,
#'     `"millions_gbp"`, `"currency_per_gbp"`, `"index"`, `"count"`).}
#'   \item{start_date}{Date. Earliest available observation date.}
#'   \item{seasonal_adjustment}{Character or `NA`. `"SA"`, `"NSA"`, or
#'     `NA` if not applicable.}
#'   \item{helper}{Character. Convenience function in the package that
#'     wraps the series.}
#' }
#'
#' @source <https://www.bankofengland.co.uk/boeapps/database/>
#' @examples
#' head(boe_series)
#' table(boe_series$category)
"boe_series"

#' Download BoE Anderson-Sleath fitted yield curves
#'
#' Downloads the Bank of England's published fitted yield curves at all
#' maturities (typically 0.5 to 25 or 40 years) using the Anderson and
#' Sleath (2001) smoothing methodology. Five curves are supported:
#' nominal gilt, real (index-linked) gilt, implied inflation, overnight
#' index swap (OIS), and the commercial bank liability curve (BLC).
#'
#' Each curve is published in two segments. The default
#' `segment = "standard"` returns the full maturity spectrum in half-year
#' steps (0.5 years out to 25 or 40). `segment = "short"` returns the
#' short end of the curve in monthly steps (one month out to five years),
#' which the Bank fits separately and which is the segment most relevant
#' to near-term policy-rate and money-market analysis. The short end is
#' available for every curve in the latest month, and historically wherever
#' the BoE published it (e.g. OIS short-end data begins later than the OIS
#' standard curve); periods without a short-end sheet are skipped.
#'
#' By default (`from = NULL`, `to = NULL`, `frequency = "daily"`) returns
#' the latest published month of daily data, matching the behaviour of
#' earlier releases of this package. Setting `from`, `to`, or `frequency`
#' switches to the BoE's full archive, which goes back to 1979 for nominal
#' gilts, 1985 for real, 2000 for BLC, and 2009 for OIS.
#'
#' @param curve Character. Which curve to fetch. One of `"nominal"`,
#'   `"real"`, `"inflation"`, `"ois"`, or `"blc"`. Defaults to
#'   `"nominal"`. The commercial bank liability curve (`"blc"`) is only
#'   published in the historical archive zip, so requests for it always
#'   route through the archive path regardless of `from` / `to`.
#' @param measure Character. `"spot"` (default) or `"forward"`.
#' @param segment Character. `"standard"` (default) for the full maturity
#'   spectrum in half-year steps, or `"short"` for the separately fitted
#'   short end in monthly steps (one month to five years).
#' @param frequency Character. `"daily"` (default) or `"monthly"`. Monthly
#'   archives are end-of-month observations and are much smaller files.
#' @param from,to Date or character ("YYYY-MM-DD"). Optional inclusive
#'   bounds. When either is set, the function uses the BoE archive zip
#'   (multi-decade history) and filters by date.
#' @param cache Logical. Use cached download if available and within the
#'   TTL window (default `TRUE`).
#' @param cache_ttl_h Numeric. Cache time-to-live in hours. When `NULL`
#'   (default) the TTL is 24 hours for the latest-month zip and 720 hours
#'   (30 days) for archive zips.
#'
#' @return A `boe_tbl` data frame with columns:
#'   \describe{
#'     \item{date}{Date. Observation date.}
#'     \item{maturity_years}{Numeric. Maturity in years.}
#'     \item{rate_pct}{Numeric. Yield or implied rate (percent).}
#'   }
#'
#' @details
#' Requires the \pkg{readxl} package. Data is published as Excel workbooks
#' inside zip archives at
#' \url{https://www.bankofengland.co.uk/statistics/yield-curves}. Each
#' archive zip contains multiple per-period workbooks; this function
#' concatenates them transparently.
#'
#' @references
#' Anderson, N. and Sleath, J. (2001). New estimates of the UK real
#' and nominal yield curves. \emph{Bank of England Working Paper No.
#' 126.} \url{https://www.bankofengland.co.uk/working-paper/2001/new-estimates-of-the-uk-real-and-nominal-yield-curves}
#'
#' @source <https://www.bankofengland.co.uk/statistics/yield-curves>
#'
#' @examples
#' \donttest{
#' if (requireNamespace("readxl", quietly = TRUE)) {
#'   op <- options(boe.cache_dir = tempdir())
#'   # Latest nominal spot curve at all maturities (default behaviour)
#'   curve <- boe_curve(curve = "nominal", measure = "spot")
#'   head(curve)
#'
#'   # Short end of the nominal forward curve (monthly steps to 5 years)
#'   se <- boe_curve(curve = "nominal", measure = "forward",
#'                   segment = "short")
#'   range(se$maturity_years)
#'   options(op)
#' }
#' }
#'
#' \dontrun{
#' # Historical archive: multi-decade downloads, so not run automatically.
#' # 10-year nominal spot back to 2010:
#' long <- boe_curve(curve = "nominal", from = "2010-01-01")
#'
#' # End-of-month real curve since 1990:
#' real_m <- boe_curve(curve = "real", frequency = "monthly",
#'                     from = "1990-01-01")
#' }
#'
#' @family interest rates
#' @export
boe_curve <- function(curve       = c("nominal", "real", "inflation", "ois", "blc"),
                      measure     = c("spot", "forward"),
                      segment     = c("standard", "short"),
                      frequency   = c("daily", "monthly"),
                      from        = NULL,
                      to          = NULL,
                      cache       = TRUE,
                      cache_ttl_h = NULL) {

  curve     <- match.arg(curve)
  measure   <- match.arg(measure)
  segment   <- match.arg(segment)
  frequency <- match.arg(frequency)

  if (!requireNamespace("readxl", quietly = TRUE)) {
    cli::cli_abort(c(
      "Package {.pkg readxl} is required to parse BoE yield curve data.",
      "i" = "Install it with {.code install.packages(\"readxl\")}."
    ))
  }

  from_d <- parse_date_arg(from, "from")
  to_d   <- parse_date_arg(to,   "to")
  if (!is.null(from_d) && !is.null(to_d) && from_d > to_d) {
    cli::cli_abort("{.arg from} ({.val {from_d}}) must be before {.arg to} ({.val {to_d}}).")
  }

  # BLC is only published in the archive zips, not in latest-yield-curve-data.zip,
  # so always route BLC through the archive path.
  use_archive <- curve == "blc" ||
                 !is.null(from_d) || !is.null(to_d) ||
                 frequency != "daily"

  if (use_archive) {
    ttl <- cache_ttl_h %||% 720
    arc <- download_yield_archive(curve, frequency,
                                  cache = cache, cache_ttl_h = ttl)
    xls_paths <- extract_archive_excels(arc$path, curve, frequency)
    out <- parse_yield_workbooks(xls_paths, measure = measure, curve = curve,
                                 segment = segment)
    source_url   <- arc$url
    source_label <- "archive"
  } else {
    ttl <- cache_ttl_h %||% 24
    zip_path <- download_yield_zip(cache = cache, cache_ttl_h = ttl)
    xls_path <- extract_yield_excel(zip_path, curve)
    out <- parse_yield_workbooks(xls_path, measure = measure, curve = curve,
                                 segment = segment)
    source_url   <- yield_zip_url()
    source_label <- "latest"
  }

  if (!is.null(from_d)) out <- out[out$date >= from_d, , drop = FALSE]
  if (!is.null(to_d))   out <- out[out$date <= to_d,   , drop = FALSE]
  rownames(out) <- NULL

  if (nrow(out) == 0L) {
    cli::cli_warn(c(
      "No observations after applying date filter.",
      "i" = "Requested {.val {from_d}} to {.val {to_d}}."
    ))
  }

  query <- list(
    series_codes  = sprintf("AS_%s_%s%s", toupper(curve), toupper(measure),
                            if (segment == "short") "_SHORT" else ""),
    from          = if (nrow(out)) min(out$date) else from_d,
    to            = if (nrow(out)) max(out$date) else to_d,
    frequency     = frequency,
    segment       = segment,
    function_name = "boe_curve",
    source_url    = source_url,
    source        = source_label
  )

  cli::cli_progress_done()
  new_boe_tbl(out, query = query)
}


#' @noRd
yield_zip_url <- function() {
  "https://www.bankofengland.co.uk/-/media/boe/files/statistics/yield-curves/latest-yield-curve-data.zip"
}


#' Download the latest-month yield curve zip
#' @noRd
download_yield_zip <- function(cache = TRUE, cache_ttl_h = 24) {
  cache_dir <- boe_cache_dir()
  dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
  cache_file <- file.path(cache_dir, "yield-curve-latest.zip")

  if (cache && file.exists(cache_file)) {
    age_h <- as.numeric(difftime(Sys.time(),
                                 file.info(cache_file)$mtime,
                                 units = "hours"))
    if (age_h < cache_ttl_h) {
      cli::cli_progress_step("Using cached yield curve archive")
      return(cache_file)
    }
  }

  cli::cli_progress_step("Downloading yield curve archive from Bank of England")

  tryCatch(
    httr2::request(yield_zip_url()) |>
      httr2::req_user_agent("boe R package (https://github.com/charlescoverdale/boe)") |>
      httr2::req_timeout(180) |>
      httr2::req_retry(
        max_tries    = 3,
        is_transient = function(resp) httr2::resp_status(resp) %in% c(429L, 503L)
      ) |>
      httr2::req_perform(path = cache_file),
    error = function(e) {
      if (file.exists(cache_file)) unlink(cache_file)
      cli::cli_abort(c(
        "Download failed.",
        "i" = "Check your internet connection.",
        "x" = conditionMessage(e)
      ))
    }
  )

  cache_file
}


#' Extract a single curve Excel file from the latest-month zip
#'
#' The latest-month zip contains daily files for every curve type. Pick the
#' one matching `curve`.
#'
#' @noRd
extract_yield_excel <- function(zip_path, curve) {
  files_in_zip <- utils::unzip(zip_path, list = TRUE)$Name

  pattern <- switch(curve,
    nominal   = "GLC Nominal",
    real      = "GLC Real",
    inflation = "GLC Inflation",
    ois       = "OIS",
    blc       = "BLC Nominal"
  )

  candidates <- files_in_zip[
    grepl(pattern, files_in_zip, fixed = TRUE) &
    grepl("daily", files_in_zip, ignore.case = TRUE) &
    grepl("\\.xlsx?$", files_in_zip, ignore.case = TRUE)
  ]

  if (length(candidates) == 0L) {
    cli::cli_abort(c(
      "Could not find {.val {curve}} daily Excel file in BoE archive.",
      "i" = "Files in archive: {.val {files_in_zip}}"
    ))
  }

  out_dir <- tempfile("boe_yield_")
  dir.create(out_dir, recursive = TRUE)
  utils::unzip(zip_path,
               files     = candidates[1L],
               exdir     = out_dir,
               junkpaths = TRUE)
  file.path(out_dir, basename(candidates[1L]))
}


#' Parse one or more BoE yield-curve workbooks and concatenate.
#'
#' Each workbook has the same internal structure regardless of period:
#' standard sheets named like "4. spot curve" / "2. fwd curve" and
#' short-end sheets named like "3. spot, short end" / "1. fwds, short end"
#' (older workbooks infix the curve name, e.g. "4. nominal spot curve").
#' The maturity row is normally row 4 (years), but older workbooks
#' (pre-2007) sometimes shift it. We use content-based detection: scan
#' rows 3 to 6 for the row whose first non-empty entries are monotonically
#' increasing positive numerics.
#'
#' @param paths Character vector of one or more workbook paths.
#' @param segment `"standard"` or `"short"`; selects which curve sheet to read.
#' @noRd
parse_yield_workbooks <- function(paths, measure, curve, segment = "standard") {
  parts <- lapply(paths, parse_yield_excel_one,
                  measure = measure, curve = curve, segment = segment)
  parts <- parts[vapply(parts, function(p) !is.null(p) && nrow(p) > 0L, logical(1))]

  if (length(parts) == 0L) {
    if (segment == "short") {
      cli::cli_abort(c(
        "No short-end {.val {measure}} data found for the {.val {curve}} curve.",
        "i" = "The BoE does not publish a short end for every curve in every period."
      ))
    }
    cli::cli_abort("Could not parse any data from {.val {curve}} workbooks.")
  }

  out <- do.call(rbind, parts)
  out <- out[!duplicated(out[, c("date", "maturity_years")]), , drop = FALSE]
  out <- out[order(out$date, out$maturity_years), , drop = FALSE]
  rownames(out) <- NULL
  out
}


#' Parse a single yield-curve workbook
#' @noRd
parse_yield_excel_one <- function(path, measure, curve, segment = "standard") {
  sheets <- readxl::excel_sheets(path)
  sheets_trim <- trimws(sheets)

  pattern <- yield_sheet_pattern(measure, segment)
  hits <- grep(pattern, sheets_trim, ignore.case = TRUE)
  if (length(hits) == 0L) {
    # A missing short-end sheet is expected for periods/curves where the
    # BoE never published one (e.g. early OIS); skip the workbook quietly
    # and let the caller decide if *no* workbook yielded data. A missing
    # standard sheet is unexpected, so surface it.
    if (segment != "short") {
      cli::cli_warn(c(
        "Could not find a {.val {measure}} curve sheet in {.file {basename(path)}}.",
        "i" = "Available sheets: {.val {sheets}}"
      ))
    }
    return(NULL)
  }
  sheet_name <- sheets[hits[1L]]

  raw <- suppressMessages(
    readxl::read_excel(path, sheet = sheet_name, col_names = FALSE)
  )
  raw <- as.data.frame(raw, stringsAsFactors = FALSE)

  if (nrow(raw) < 6L || ncol(raw) < 2L) {
    cli::cli_warn("Sheet {.val {sheet_name}} in {.file {basename(path)}} is too small to parse.")
    return(NULL)
  }

  mat_row <- detect_maturity_row(raw)
  if (is.na(mat_row)) {
    cli::cli_warn("Could not detect maturity row in {.file {basename(path)}}.")
    return(NULL)
  }
  maturities <- suppressWarnings(as.numeric(unlist(raw[mat_row, -1L])))
  good_cols  <- !is.na(maturities) & maturities > 0
  if (!any(good_cols)) return(NULL)
  maturities <- maturities[good_cols]

  data_start <- mat_row + 2L
  body <- raw[data_start:nrow(raw), c(TRUE, good_cols), drop = FALSE]

  date_col <- body[[1L]]
  if (inherits(date_col, "POSIXt") || inherits(date_col, "Date")) {
    dates <- as.Date(date_col)
    keep  <- !is.na(dates)
  } else {
    serials <- suppressWarnings(as.numeric(date_col))
    keep    <- !is.na(serials)
    dates   <- as.Date(serials, origin = "1899-12-30")
  }
  body  <- body[keep, , drop = FALSE]
  dates <- dates[keep]

  if (length(dates) == 0L) return(NULL)

  vals <- suppressWarnings(matrix(
    as.numeric(as.matrix(body[, -1L, drop = FALSE])),
    nrow = nrow(body),
    ncol = length(maturities)
  ))

  long <- data.frame(
    date           = rep(dates, times = ncol(vals)),
    maturity_years = rep(maturities, each = nrow(vals)),
    rate_pct       = as.numeric(vals),
    stringsAsFactors = FALSE
  )
  long[!is.na(long$rate_pct), , drop = FALSE]
}


#' Build the regex that selects a curve sheet by measure and segment.
#'
#' Sheet names are anchored at the end so the optional curve-name infix in
#' older workbooks (e.g. "4. nominal spot curve", "1. nominal fwds, short
#' end") still matches. Standard forward sheets are singular ("fwd curve")
#' while short-end forward sheets are plural ("fwds, short end"); the
#' patterns tolerate both and an optional comma.
#'
#' @noRd
yield_sheet_pattern <- function(measure, segment) {
  if (segment == "short") {
    if (measure == "spot") "spot,? short end$" else "fwds?,? short end$"
  } else {
    if (measure == "spot") "spot curve$" else "fwd curve$"
  }
}


#' Detect which row holds the maturity grid (years).
#'
#' Default fast path is row 4 (modern BoE workbooks). Fall back to
#' scanning rows 3 to 6 for the row whose first non-empty values are
#' increasing positive numerics starting near 0.5.
#'
#' @noRd
detect_maturity_row <- function(raw) {
  fast <- suppressWarnings(as.numeric(unlist(raw[4L, -1L])))
  if (sum(!is.na(fast) & fast > 0) >= 5L) return(4L)

  for (r in c(3L, 5L, 6L)) {
    if (r > nrow(raw)) next
    cand <- suppressWarnings(as.numeric(unlist(raw[r, -1L])))
    cand <- cand[!is.na(cand)]
    if (length(cand) >= 5L &&
        all(cand > 0) &&
        all(diff(cand[seq_len(min(length(cand), 10L))]) >= 0)) {
      return(r)
    }
  }
  NA_integer_
}


#' Null-coalescing operator
#' @noRd
`%||%` <- function(a, b) if (is.null(a)) b else a

# Tests for boe_curve() and boe_curve_panel().
# Network-dependent paths are skipped on CRAN and offline.

# ---- argument validation (offline, no network) ------------------------------

test_that("boe_curve rejects invalid curve / measure / frequency", {
  expect_error(boe_curve(curve = "foo"))
  expect_error(boe_curve(measure = "par"))
  expect_error(boe_curve(frequency = "weekly"))
})

test_that("boe_curve rejects from > to", {
  expect_error(
    boe_curve(curve = "nominal", from = "2020-01-01", to = "2019-01-01"),
    regexp = "must be before"
  )
})

test_that("boe_curve_panel rejects non-positive maturities", {
  expect_error(boe_curve_panel(maturities = c(-1, 5)))
  expect_error(boe_curve_panel(maturities = c(0, 5)))
})

test_that("boe_curve and boe_curve_panel reject invalid segment", {
  expect_error(boe_curve(segment = "frontend"))
  expect_error(boe_curve_panel(segment = "frontend"))
})

# ---- short-end sheet selection (offline, pure function) ----------------------

test_that("yield_sheet_pattern selects sheets by measure and segment", {
  # Standard sheets (modern + curve-name-infixed older layout).
  expect_match("4. spot curve",         yield_sheet_pattern("spot", "standard"))
  expect_match("4. nominal spot curve", yield_sheet_pattern("spot", "standard"))
  expect_match("2. fwd curve",          yield_sheet_pattern("forward", "standard"))

  # Short-end sheets: spot uses "spot, short end"; forward is plural "fwds".
  expect_match("3. spot, short end",         yield_sheet_pattern("spot", "short"))
  expect_match("3. nominal spot, short end", yield_sheet_pattern("spot", "short"))
  expect_match("1. fwds, short end",         yield_sheet_pattern("forward", "short"))
  expect_match("1. nominal fwds, short end", yield_sheet_pattern("forward", "short"))
})

test_that("yield_sheet_pattern does not cross-match segments", {
  expect_false(grepl(yield_sheet_pattern("spot", "standard"),
                     "3. spot, short end", ignore.case = TRUE))
  expect_false(grepl(yield_sheet_pattern("spot", "short"),
                     "4. spot curve", ignore.case = TRUE))
  expect_false(grepl(yield_sheet_pattern("forward", "standard"),
                     "1. fwds, short end", ignore.case = TRUE))
})

# ---- panel pillar labelling (offline, mocked boe_curve) ---------------------

test_that("boe_curve_panel keeps labels aligned when a pillar is dropped", {
  fake_long <- new_boe_tbl(
    data.frame(
      date           = rep(as.Date(c("2020-01-01", "2020-01-02")), each = 3),
      maturity_years = rep(c(1, 2, 5), times = 2),
      rate_pct       = c(1.1, 2.2, 5.5, 1.2, 2.3, 5.6)
    ),
    query = list(function_name = "boe_curve", source = "latest")
  )
  testthat::local_mocked_bindings(boe_curve = function(...) fake_long)

  # Pillar 10 is absent from the (1, 2, 5) grid, so it must drop without
  # shifting the labels of the surviving columns.
  expect_warning(
    panel <- boe_curve_panel(curve = "nominal", maturities = c(1, 2, 5, 10)),
    regexp = "not on the"
  )
  expect_named(panel, c("date", "m1", "m2", "m5"))
  expect_equal(panel$m1[panel$date == as.Date("2020-01-01")], 1.1)
  expect_equal(panel$m5[panel$date == as.Date("2020-01-01")], 5.5)
})

# ---- network-dependent: short end -------------------------------------------

test_that("boe_curve fetches the short end with monthly maturities", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("readxl")

  op <- options(boe.cache_dir = tempfile("boe_short_"))
  on.exit(options(op), add = TRUE)

  out <- boe_curve(curve = "nominal", measure = "spot", segment = "short")

  expect_s3_class(out, "boe_tbl")
  expect_gt(nrow(out), 50L)
  expect_lt(min(out$maturity_years), 0.5)    # sub-6-month points exist
  expect_lte(max(out$maturity_years), 5.01)  # short end caps near 5 years

  q <- attr(out, "boe_query")
  expect_equal(q$segment, "short")
  expect_match(q$series_codes, "AS_NOMINAL_SPOT_SHORT")
})

test_that("short end caps lower than standard; forward short end parses", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("readxl")

  op <- options(boe.cache_dir = tempfile("boe_short2_"))
  on.exit(options(op), add = TRUE)

  std   <- boe_curve(curve = "nominal", measure = "spot", segment = "standard")
  short <- boe_curve(curve = "nominal", measure = "spot", segment = "short")
  expect_lt(max(short$maturity_years), max(std$maturity_years))

  fwd <- boe_curve(curve = "nominal", measure = "forward", segment = "short")
  expect_gt(nrow(fwd), 50L)
})

test_that("OIS short end skips periods without a short-end sheet", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("readxl")

  op <- options(boe.cache_dir = tempfile("boe_ois_short_"))
  on.exit(options(op), add = TRUE)

  # OIS publishes no short end for 2009-2015; requesting from 2009 must skip
  # those workbooks cleanly rather than error, and still return later data.
  out <- boe_curve(curve = "ois", measure = "spot", segment = "short",
                   from = "2009-01-01")
  expect_s3_class(out, "boe_tbl")
  expect_gt(nrow(out), 0L)
  expect_gt(min(out$date), as.Date("2010-01-01"))
})

test_that("boe_curve_panel short segment returns short-end pillars", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("readxl")

  op <- options(boe.cache_dir = tempfile("boe_panel_short_"))
  on.exit(options(op), add = TRUE)

  panel <- boe_curve_panel(curve = "nominal", measure = "spot",
                           segment = "short", maturities = c(1, 2, 5))
  expect_s3_class(panel, "boe_tbl")
  expect_named(panel, c("date", "m1", "m2", "m5"))
  expect_gt(nrow(panel), 10L)
})

# ---- archive URL registry (offline) -----------------------------------------

test_that("yield_archive_registry has 5 curves x 2 frequencies", {
  reg <- yield_archive_registry()
  expect_named(reg, c("daily", "monthly"))
  expect_named(reg$daily,
               c("nominal", "real", "inflation", "ois", "blc"),
               ignore.order = TRUE)
  expect_named(reg$monthly,
               c("nominal", "real", "inflation", "ois", "blc"),
               ignore.order = TRUE)
})

test_that("yield_archive_url constructs valid URLs", {
  u <- yield_archive_url("nominal", "daily")
  expect_match(u, "^https://www.bankofengland.co.uk/.*\\.zip$")
  expect_match(u, "glcnominalddata.zip$")

  u2 <- yield_archive_url("ois", "monthly")
  expect_match(u2, "oismonthedata.zip$")

  u3 <- yield_archive_url("blc", "daily")
  expect_match(u3, "blcnomddata.zip$")
})

test_that("yield_archive_url errors on unknown curve", {
  expect_error(yield_archive_url("treasury", "daily"))
})

# ---- maturity-row detection (offline, synthetic) ----------------------------

test_that("detect_maturity_row finds row 4 in modern layout", {
  raw <- data.frame(
    matrix(c(
      "title", NA, NA, NA, NA, NA,
      "Maturity ", NA, NA, NA, NA, NA,
      "months:", "6", "12", "24", "36", "60",
      "years:", 0.5, 1, 2, 3, 5,
      NA, NA, NA, NA, NA, NA,
      "1970-01-31", 1.0, 1.1, 1.2, 1.3, 1.4
    ), nrow = 6, byrow = TRUE),
    stringsAsFactors = FALSE
  )
  expect_equal(detect_maturity_row(raw), 4L)
})

test_that("detect_maturity_row falls back when row 4 is empty", {
  raw <- data.frame(
    matrix(c(
      "title", NA, NA, NA, NA, NA,
      "years:", 0.5, 1, 2, 3, 5,           # row 2 holds maturities
      NA, NA, NA, NA, NA, NA,
      NA, NA, NA, NA, NA, NA,              # row 4 is empty
      NA, NA, NA, NA, NA, NA,
      "1970-01-31", 1.0, 1.1, 1.2, 1.3, 1.4
    ), nrow = 6, byrow = TRUE),
    stringsAsFactors = FALSE
  )
  out <- detect_maturity_row(raw)
  expect_true(is.na(out) || out %in% c(3L, 5L, 6L))
})

# ---- network-dependent: latest month (default behaviour) --------------------

test_that("boe_curve fetches and parses the nominal spot curve", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("readxl")

  op <- options(boe.cache_dir = tempfile("boe_curve_"))
  on.exit(options(op), add = TRUE)

  out <- boe_curve(curve = "nominal", measure = "spot")

  expect_s3_class(out, "boe_tbl")
  expect_true(all(c("date", "maturity_years", "rate_pct") %in% names(out)))
  expect_gt(nrow(out), 50L)
  expect_true(all(out$maturity_years >= 0.5))
  expect_true(all(is.finite(out$rate_pct)))

  q <- attr(out, "boe_query")
  expect_equal(q$function_name, "boe_curve")
  expect_equal(q$source, "latest")
  expect_match(q$series_codes, "AS_NOMINAL_SPOT")
})

test_that("boe_curve forward measure works for nominal", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("readxl")

  op <- options(boe.cache_dir = tempfile("boe_curve_fwd_"))
  on.exit(options(op), add = TRUE)

  out <- boe_curve(curve = "nominal", measure = "forward")
  expect_s3_class(out, "boe_tbl")
  expect_gt(nrow(out), 50L)
})

test_that("boe_curve fetches inflation and OIS curves", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("readxl")

  op <- options(boe.cache_dir = tempfile("boe_curve_misc_"))
  on.exit(options(op), add = TRUE)

  inflation <- boe_curve(curve = "inflation", measure = "spot")
  expect_s3_class(inflation, "boe_tbl")
  expect_gt(nrow(inflation), 0L)

  ois <- boe_curve(curve = "ois", measure = "spot")
  expect_s3_class(ois, "boe_tbl")
  expect_gt(nrow(ois), 0L)
})

# ---- network-dependent: archive paths ---------------------------------------

test_that("boe_curve fetches monthly archive for OIS (smallest zip)", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("readxl")

  op <- options(boe.cache_dir = tempfile("boe_curve_arc_"))
  on.exit(options(op), add = TRUE)

  out <- boe_curve(curve = "ois", measure = "spot",
                   frequency = "monthly", from = "2015-01-01")

  expect_s3_class(out, "boe_tbl")
  expect_gt(nrow(out), 100L)
  expect_true(min(out$date) >= as.Date("2015-01-01"))

  q <- attr(out, "boe_query")
  expect_equal(q$source, "archive")
  expect_equal(q$frequency, "monthly")
})

test_that("boe_curve_panel returns wide format with requested pillars", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("readxl")

  op <- options(boe.cache_dir = tempfile("boe_panel_"))
  on.exit(options(op), add = TRUE)

  # OIS only extends to ~5y so test pillars stay within that range
  panel <- boe_curve_panel(
    curve     = "ois",
    measure   = "spot",
    frequency = "monthly",
    from      = "2015-01-01",
    maturities = c(1, 3, 5)
  )

  expect_s3_class(panel, "boe_tbl")
  expect_named(panel, c("date", "m1", "m3", "m5"))
  expect_gt(nrow(panel), 100L)
  expect_true(all(is.finite(panel$m5) | is.na(panel$m5)))
})

test_that("boe_curve fetches BLC daily latest", {
  testthat::skip_on_cran()
  testthat::skip_if_offline()
  testthat::skip_if_not_installed("readxl")

  op <- options(boe.cache_dir = tempfile("boe_blc_"))
  on.exit(options(op), add = TRUE)

  out <- boe_curve(curve = "blc", measure = "spot")
  expect_s3_class(out, "boe_tbl")
  expect_gt(nrow(out), 50L)
})

#' Inspect the local BoE cache
#'
#' Reports the cache directory, number of cached files, total size,
#' and oldest / newest modification timestamps. Prints a short summary
#' and returns the underlying values invisibly.
#'
#' The cache directory defaults to `tools::R_user_dir("boe", "cache")`
#' and can be overridden with `options(boe.cache_dir = ...)`.
#'
#' @return Invisibly, a list with elements:
#'   \describe{
#'     \item{path}{Character. Cache directory.}
#'     \item{n_files}{Integer. Number of cached files.}
#'     \item{total_size_bytes}{Numeric. Total size on disk (bytes).}
#'     \item{oldest}{POSIXct. Modification time of oldest file (or
#'       `NA` if cache is empty).}
#'     \item{newest}{POSIXct. Modification time of newest file (or
#'       `NA` if cache is empty).}
#'   }
#'
#' @examples
#' \donttest{
#' op <- options(boe.cache_dir = tempdir())
#' boe_cache_info()
#' options(op)
#' }
#'
#' @family cache
#' @seealso [clear_cache()]
#' @export
boe_cache_info <- function() {
  dir   <- boe_cache_dir()
  files <- if (dir.exists(dir)) {
    list.files(dir, full.names = TRUE)
  } else {
    character(0L)
  }
  n      <- length(files)
  info   <- if (n) file.info(files) else NULL
  size   <- if (n) sum(info$size, na.rm = TRUE) else 0
  oldest <- if (n) min(info$mtime, na.rm = TRUE) else as.POSIXct(NA)
  newest <- if (n) max(info$mtime, na.rm = TRUE) else as.POSIXct(NA)

  msg <- c(
    "BoE cache",
    "*" = sprintf("Path:  %s", dir),
    "*" = sprintf("Files: %d", n),
    "*" = sprintf("Size:  %s", format_cache_size(size))
  )
  if (n) {
    msg <- c(msg, "*" = sprintf("Range: %s to %s",
                                format(oldest), format(newest)))
  }
  cli::cli_inform(msg)

  invisible(list(
    path             = dir,
    n_files          = n,
    total_size_bytes = as.numeric(size),
    oldest           = oldest,
    newest           = newest
  ))
}

#' Format a byte count as a human-readable string
#' @noRd
format_cache_size <- function(bytes) {
  if (is.na(bytes) || bytes < 0) return(NA_character_)
  if (bytes < 1024)        return(sprintf("%d B", as.integer(bytes)))
  if (bytes < 1024^2)      return(sprintf("%.1f KB", bytes / 1024))
  if (bytes < 1024^3)      return(sprintf("%.1f MB", bytes / 1024^2))
  sprintf("%.1f GB", bytes / 1024^3)
}

# Tests for boe_cache_info().

test_that("boe_cache_info() reports an empty cache without error", {
  d <- tempfile("boe_cache_empty_")
  dir.create(d)
  op <- options(boe.cache_dir = d)
  on.exit(options(op), add = TRUE)

  info <- suppressMessages(boe_cache_info())
  expect_equal(info$path, d)
  expect_equal(info$n_files, 0L)
  expect_equal(info$total_size_bytes, 0)
  expect_true(is.na(info$oldest))
  expect_true(is.na(info$newest))
})

test_that("boe_cache_info() reports a populated cache", {
  d <- tempfile("boe_cache_pop_")
  dir.create(d)
  writeLines("hello", file.path(d, "a.csv"))
  writeLines("world world", file.path(d, "b.csv"))
  op <- options(boe.cache_dir = d)
  on.exit(options(op), add = TRUE)

  info <- suppressMessages(boe_cache_info())
  expect_equal(info$n_files, 2L)
  expect_gt(info$total_size_bytes, 0)
  expect_s3_class(info$oldest, "POSIXct")
  expect_s3_class(info$newest, "POSIXct")
})

test_that("boe_cache_info() handles a non-existent cache directory", {
  d <- tempfile("boe_cache_missing_")
  op <- options(boe.cache_dir = d)
  on.exit(options(op), add = TRUE)

  info <- suppressMessages(boe_cache_info())
  expect_equal(info$path, d)
  expect_equal(info$n_files, 0L)
})

test_that("format_cache_size handles common ranges", {
  expect_equal(format_cache_size(0), "0 B")
  expect_equal(format_cache_size(1023), "1023 B")
  expect_equal(format_cache_size(2048), "2.0 KB")
  expect_equal(format_cache_size(5 * 1024^2), "5.0 MB")
  expect_equal(format_cache_size(2 * 1024^3), "2.0 GB")
})

---
title: "Anderson-Sleath yield curves: latest and historical"
author: "Charles Coverdale"
date: "`r format(Sys.Date(), '%d %B %Y')`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Anderson-Sleath yield curves: latest and historical}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
library(boe)
not_on_cran <- identical(Sys.getenv("NOT_CRAN"), "true")
knitr::opts_chunk$set(
  collapse = TRUE,
  comment  = "#>",
  fig.width  = 7,
  fig.height = 4.5,
  eval = not_on_cran
)
op <- options(boe.cache_dir = tempfile("boe_vignette_"))
```

The Bank of England publishes daily fitted yield curves at all maturities
using the Anderson and Sleath (2001) smoothing methodology. Five curves
are produced: nominal gilt, real (index-linked) gilt, implied inflation,
overnight index swap (OIS), and the commercial bank liability curve
(BLC). Each is available in spot and instantaneous-forward form, and in
two segments: the *standard* curve (half-year maturity steps out to 25 or
40 years) and the separately fitted *short end* (monthly steps from one
month to five years), selected with `segment = "short"`.

The default behaviour of `boe_curve()` returns the latest published
month, matching what most analysts need when they reach for "today's
curve". Pass `from`, `to`, or `frequency = "monthly"` and the function
switches to the BoE historical archive, which extends back as far as
1979 for nominal gilts.

## Latest published month

```{r latest}
latest <- boe_curve(curve = "nominal", measure = "spot")
range(latest$date)
range(latest$maturity_years)
```

`boe_curve()` returns a long-format `boe_tbl` with one row per
(date, maturity) pair. Provenance is attached as an attribute:

```{r latest-provenance}
attr(latest, "boe_query")$source
attr(latest, "boe_query")$series_codes
```

## Historical: 10-year nominal spot rate since 2000

For time-series work, `boe_curve_panel()` reshapes the long format into
a wide panel with one column per pillar maturity. End-of-month frequency
is plenty for multi-decade work and keeps the download small.

```{r hist-10y}
panel <- boe_curve_panel(
  curve     = "nominal",
  measure   = "spot",
  frequency = "monthly",
  from      = "2000-01-01",
  maturities = c(2, 5, 10, 20)
)
head(panel)
```

```{r hist-10y-plot, fig.alt = "10-year UK nominal spot rate, 2000 to present"}
if (requireNamespace("ggplot2", quietly = TRUE)) {
  ggplot2::ggplot(panel, ggplot2::aes(date, m10)) +
    ggplot2::geom_line(colour = "#1f77b4") +
    ggplot2::labs(
      title    = "UK 10-year nominal spot rate",
      subtitle = "End of month, Anderson-Sleath fitted",
      x = NULL, y = "Per cent"
    ) +
    ggplot2::theme_minimal()
}
```

## 5y5y forward implied inflation

The 5y5y forward inflation rate (the average implied inflation rate over
the second five-year horizon, five years from now) is a textbook
medium-term inflation expectations measure. It comes straight off the
implied-inflation forward curve.

```{r five-five}
inflation_fwd <- boe_curve_panel(
  curve     = "inflation",
  measure   = "forward",
  frequency = "monthly",
  from      = "2010-01-01",
  maturities = c(5, 10)
)

inflation_fwd$five_y_five_y <- (inflation_fwd$m10 * 10 -
                                inflation_fwd$m5  *  5) / 5
head(inflation_fwd[, c("date", "m5", "m10", "five_y_five_y")])
```

```{r five-five-plot, fig.alt = "UK 5y5y forward implied inflation, 2010 to present"}
if (requireNamespace("ggplot2", quietly = TRUE)) {
  ggplot2::ggplot(inflation_fwd, ggplot2::aes(date, five_y_five_y)) +
    ggplot2::geom_line(colour = "#d62728") +
    ggplot2::geom_hline(yintercept = 2.0, linetype = "dashed",
                        colour = "grey40") +
    ggplot2::annotate("text", x = max(inflation_fwd$date),
                      y = 2.0, label = "2% target", hjust = 1, vjust = -0.5,
                      colour = "grey40", size = 3) +
    ggplot2::labs(
      title    = "UK 5y5y forward implied inflation",
      subtitle = "End of month, derived from the BoE implied-inflation forward curve",
      x = NULL, y = "Per cent per annum"
    ) +
    ggplot2::theme_minimal()
}
```

## OIS curve evolution across the rate cycle

The OIS curve gives a market-implied path for Bank Rate. Comparing OIS
spot pillars at MPC decision dates shows how expectations shifted
through the 2022 to 2024 hiking cycle.

```{r ois-cycle}
ois <- boe_curve_panel(
  curve     = "ois",
  measure   = "spot",
  frequency = "monthly",
  from      = "2020-01-01",
  maturities = c(0.5, 1, 2, 5)
)

mpc <- boe_mpc_decisions(from = "2020-01-01")
mpc <- data.frame(date = mpc$date, bank_rate = mpc$new_rate_pct)

merged <- merge(ois, mpc, by = "date", all.x = TRUE)
merged$bank_rate <- as.numeric(merged$bank_rate)

# carry the bank rate forward between MPC dates
for (i in seq_along(merged$bank_rate)) {
  if (i > 1 && is.na(merged$bank_rate[i])) {
    merged$bank_rate[i] <- merged$bank_rate[i - 1]
  }
}
tail(merged)
```

```{r ois-cycle-plot, fig.alt = "UK OIS spot pillars and Bank Rate, 2020 to present"}
if (requireNamespace("ggplot2", quietly = TRUE)) {
  long <- data.frame(
    date  = rep(merged$date, 5),
    pillar = rep(c("Bank Rate", "6m OIS", "1y OIS", "2y OIS", "5y OIS"),
                 each = nrow(merged)),
    rate  = c(merged$bank_rate, merged$m0.5, merged$m1, merged$m2, merged$m5)
  )
  long$pillar <- factor(long$pillar,
                        levels = c("Bank Rate", "6m OIS", "1y OIS",
                                   "2y OIS", "5y OIS"))

  ggplot2::ggplot(long, ggplot2::aes(date, rate, colour = pillar)) +
    ggplot2::geom_line() +
    ggplot2::labs(
      title    = "UK OIS spot pillars and Bank Rate, 2020 to present",
      subtitle = "Tightening cycle visible across all pillars",
      x = NULL, y = "Per cent", colour = NULL
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(legend.position = "bottom")
}
```

## The short end of the curve

The standard curves step in half-years from 0.5 years out. For near-term
policy and money-market work the Bank fits a separate *short end* at
monthly maturities, from one month to five years. Pass
`segment = "short"` to `boe_curve()` or `boe_curve_panel()` to reach it.

```{r short-end}
short <- boe_curve(curve = "nominal", measure = "spot", segment = "short")
range(short$maturity_years)   # monthly grid, ~1/12 to 5 years
```

The short end of the OIS *forward* curve is the cleanest market-implied
path for Bank Rate: the instantaneous forward rate at each horizon, at
monthly resolution. Here it is on the most recent published date.

```{r short-ois}
ois_short  <- boe_curve(curve = "ois", measure = "forward", segment = "short")
latest_day <- ois_short[ois_short$date == max(ois_short$date), ]
head(latest_day)
```

```{r short-ois-plot, fig.alt = "UK OIS short-end forward curve on the latest published date"}
if (requireNamespace("ggplot2", quietly = TRUE)) {
  ggplot2::ggplot(latest_day, ggplot2::aes(maturity_years, rate_pct)) +
    ggplot2::geom_line(colour = "#1f77b4") +
    ggplot2::geom_point(size = 0.8, colour = "#1f77b4") +
    ggplot2::labs(
      title    = "UK OIS instantaneous forward curve, short end",
      subtitle = paste("Market-implied Bank Rate path on",
                       format(max(ois_short$date), "%d %B %Y")),
      x = "Horizon (years)", y = "Per cent"
    ) +
    ggplot2::theme_minimal()
}
```

Short-end history goes back as far as the Bank published it: to 1979 for
nominal gilts, and from 2016 for OIS. Where a period has no short-end
sheet (early OIS, for instance), those dates are simply absent from the
result rather than causing an error.

## When to reach for the archive

| Question | Argument set |
|---|---|
| Today's curve | none (default) |
| Last few years, daily | `from = "2020-01-01"` |
| Multi-decade panel for econometrics | `frequency = "monthly", from = "1990-01-01"` |
| Near-term policy-rate path, monthly detail | `segment = "short"` |
| Commercial bank liability curve | `curve = "blc"` (always uses archive) |

Archive zips cache for 30 days by default; the latest-month zip caches
for 24 hours. Override with `cache_ttl_h` if you need to force a fresh
pull.

## References

Anderson, N. and Sleath, J. (2001). *New estimates of the UK real and
nominal yield curves.* Bank of England Working Paper No. 126.
[https://www.bankofengland.co.uk/working-paper/2001/new-estimates-of-the-uk-real-and-nominal-yield-curves](https://www.bankofengland.co.uk/working-paper/2001/new-estimates-of-the-uk-real-and-nominal-yield-curves)

```{r teardown, include = FALSE}
options(op)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_sonia.R
\name{boe_sonia}
\alias{boe_sonia}
\title{Download SONIA interest rate}
\source{
\url{https://www.bankofengland.co.uk/boeapps/database/}
}
\usage{
boe_sonia(
  from = "1997-01-02",
  to = Sys.Date(),
  frequency = c("daily", "monthly", "annual"),
  cache = TRUE
)
}
\arguments{
\item{from}{Date or character (YYYY-MM-DD). Start date. Defaults to
\code{"1997-01-02"}.}

\item{to}{Date or character (YYYY-MM-DD). End date. Defaults to today.}

\item{frequency}{Character. One of \code{"daily"} (default), \code{"monthly"}
(monthly average), or \code{"annual"} (annual average).}

\item{cache}{Logical. Use cached data if available (default \code{TRUE}).}
}
\value{
A data frame with columns:
\describe{
\item{date}{Date. Observation date.}
\item{rate_pct}{Numeric. SONIA rate (percent).}
}
}
\description{
Downloads the Sterling Overnight Index Average (SONIA), the risk-free
reference rate for sterling markets. Available daily from January 1997.
}
\examples{
\donttest{
op <- options(boe.cache_dir = tempdir())
boe_sonia(from = "2020-01-01")
options(op)
}

}
\seealso{
Other interest rates: 
\code{\link{boe_bank_rate}()},
\code{\link{boe_curve}()},
\code{\link{boe_curve_panel}()},
\code{\link{boe_yield_curve}()}
}
\concept{interest rates}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_cache_info.R
\name{boe_cache_info}
\alias{boe_cache_info}
\title{Inspect the local BoE cache}
\usage{
boe_cache_info()
}
\value{
Invisibly, a list with elements:
\describe{
\item{path}{Character. Cache directory.}
\item{n_files}{Integer. Number of cached files.}
\item{total_size_bytes}{Numeric. Total size on disk (bytes).}
\item{oldest}{POSIXct. Modification time of oldest file (or
\code{NA} if cache is empty).}
\item{newest}{POSIXct. Modification time of newest file (or
\code{NA} if cache is empty).}
}
}
\description{
Reports the cache directory, number of cached files, total size,
and oldest / newest modification timestamps. Prints a short summary
and returns the underlying values invisibly.
}
\details{
The cache directory defaults to \code{tools::R_user_dir("boe", "cache")}
and can be overridden with \code{options(boe.cache_dir = ...)}.
}
\examples{
\donttest{
op <- options(boe.cache_dir = tempdir())
boe_cache_info()
options(op)
}

}
\seealso{
\code{\link[=clear_cache]{clear_cache()}}
}
\concept{cache}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_mortgage_approvals.R
\name{boe_mortgage_approvals}
\alias{boe_mortgage_approvals}
\title{Download mortgage approvals for house purchase}
\source{
\url{https://www.bankofengland.co.uk/boeapps/database/}
}
\usage{
boe_mortgage_approvals(
  from = "1993-04-01",
  to = Sys.Date(),
  seasonally_adjusted = TRUE,
  cache = TRUE
)
}
\arguments{
\item{from}{Date or character (YYYY-MM-DD). Start date. Defaults to
\code{"1993-04-01"}.}

\item{to}{Date or character (YYYY-MM-DD). End date. Defaults to today.}

\item{seasonally_adjusted}{Logical. Return seasonally adjusted series
(default \code{TRUE}) or non-seasonally adjusted (\code{FALSE}).}

\item{cache}{Logical. Use cached data if available (default \code{TRUE}).}
}
\value{
A data frame with columns:
\describe{
\item{date}{Date. End of month.}
\item{approvals}{Numeric. Number of mortgage approvals.}
}
}
\description{
Downloads the monthly count of mortgage approvals for house purchase,
a widely watched leading indicator of housing market activity.
Available from April 1993.
}
\examples{
\donttest{
op <- options(boe.cache_dir = tempdir())
boe_mortgage_approvals(from = "2015-01-01")
options(op)
}

}
\seealso{
Other credit and housing: 
\code{\link{boe_consumer_credit}()},
\code{\link{boe_mortgage_rates}()}
}
\concept{credit and housing}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clear_cache.R
\name{clear_cache}
\alias{clear_cache}
\title{Clear locally cached Bank of England data}
\usage{
clear_cache(max_age_days = NULL)
}
\arguments{
\item{max_age_days}{Numeric or \code{NULL}. If specified, only removes files
older than this many days. If \code{NULL} (the default), removes all
cached files.}
}
\value{
Invisibly returns the number of files removed.
}
\description{
Removes cached data files downloaded from the Bank of England.
}
\examples{
\donttest{
op <- options(boe.cache_dir = tempdir())
# Remove files older than 7 days
clear_cache(max_age_days = 7)

# Remove everything
clear_cache()
options(op)
}

}
\seealso{
Other data access: 
\code{\link{boe_get}()}
}
\concept{data access}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_search.R
\name{boe_browse}
\alias{boe_browse}
\title{Browse the BoE series catalogue}
\usage{
boe_browse(category = NULL, frequency = NULL)
}
\arguments{
\item{category}{Character. Optional filter on the \code{category} column,
one or more of \code{"interest_rates"}, \code{"exchange_rates"},
\code{"mortgage_market"}, \code{"consumer_credit"}, \code{"monetary_aggregates"}.}

\item{frequency}{Character. Optional filter on the \code{frequency}
column, one or more of \code{"daily"}, \code{"monthly"}, \code{"annual"}.}
}
\value{
A data frame with the same columns as \link{boe_series}.
}
\description{
Returns the catalogue with optional category or frequency filters.
Equivalent to \code{boe_search(query = NULL, category, frequency)} but
framed as a browse / inspect action rather than a keyword search.
}
\examples{
# The whole catalogue
nrow(boe_browse())

# All exchange rate series
boe_browse(category = "exchange_rates")

# All monthly series
boe_browse(frequency = "monthly")

}
\seealso{
\code{\link[=boe_search]{boe_search()}}, \link{boe_series}

Other discovery: 
\code{\link{boe_search}()}
}
\concept{discovery}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_exchange_rate.R
\name{boe_exchange_rate}
\alias{boe_exchange_rate}
\title{Download sterling exchange rates}
\source{
\url{https://www.bankofengland.co.uk/boeapps/database/}
}
\usage{
boe_exchange_rate(
  currency = "USD",
  from = "1975-01-02",
  to = Sys.Date(),
  cache = TRUE
)
}
\arguments{
\item{currency}{Character vector. One or more currency codes. Use
\code{\link[=list_exchange_rates]{list_exchange_rates()}} to see all available currencies. Defaults to
\code{"USD"}.}

\item{from}{Date or character (YYYY-MM-DD). Start date. Defaults to
\code{"1975-01-02"}.}

\item{to}{Date or character (YYYY-MM-DD). End date. Defaults to today.}

\item{cache}{Logical. Use cached data if available (default \code{TRUE}).}
}
\value{
A data frame with columns:
\describe{
\item{date}{Date. Observation date.}
\item{currency}{Character. Currency code (e.g. \code{"USD"}).}
\item{rate}{Numeric. Units of foreign currency per GBP.}
}
}
\description{
Downloads daily spot exchange rates for sterling against major currencies
from the Bank of England. Most series available from January 1975.
}
\examples{
\donttest{
op <- options(boe.cache_dir = tempdir())
# GBP/USD since 2020
boe_exchange_rate("USD", from = "2020-01-01")

# Multiple currencies
boe_exchange_rate(c("USD", "EUR", "JPY"), from = "2020-01-01")
options(op)
}

}
\seealso{
Other exchange rates: 
\code{\link{list_exchange_rates}()}
}
\concept{exchange rates}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_mpc_votes.R
\name{boe_mpc_votes}
\alias{boe_mpc_votes}
\title{Monetary Policy Committee voting history}
\source{
\url{https://www.bankofengland.co.uk/monetary-policy}
}
\usage{
boe_mpc_votes(cache = TRUE)
}
\arguments{
\item{cache}{Logical. Use cached download if less than 24 hours old
(default \code{TRUE}). The voting workbook is small (~110 KB).}
}
\value{
A \code{boe_tbl} data frame with columns:
\describe{
\item{date}{Date. Meeting date.}
\item{member}{Character. MPC member name.}
\item{member_vote_pct}{Numeric. The Bank Rate the member voted
for (percent).}
\item{decision_pct}{Numeric. The committee's decision (percent).}
\item{dissent}{Logical. \code{TRUE} if the member's vote differed from
the committee decision.}
}
}
\description{
Downloads the Bank of England's published MPC voting record and
returns it in long format. Each row is one (meeting, member) pair
showing the rate the member voted for and whether that constituted
a dissent from the committee's decision.
}
\details{
Coverage runs from the first MPC meeting in June 1997 through the
most recent published minutes. Both current and past committee
members are included.

Requires the \pkg{readxl} package.
}
\examples{
\donttest{
if (requireNamespace("readxl", quietly = TRUE)) {
  op <- options(boe.cache_dir = tempdir())
  votes <- boe_mpc_votes()

  # Recent dissents
  recent <- subset(votes, dissent & date >= as.Date("2024-01-01"))
  head(recent)

  options(op)
}
}

}
\seealso{
\code{\link[=boe_mpc_decisions]{boe_mpc_decisions()}}

Other monetary policy: 
\code{\link{boe_mpc_decisions}()},
\code{\link{boe_mpr_forecasts}()}
}
\concept{monetary policy}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_curve_panel.R
\name{boe_curve_panel}
\alias{boe_curve_panel}
\title{Wide panel of BoE yield curve at chosen pillar maturities}
\usage{
boe_curve_panel(
  curve = c("nominal", "real", "inflation", "ois", "blc"),
  measure = c("spot", "forward"),
  segment = c("standard", "short"),
  frequency = c("daily", "monthly"),
  from = NULL,
  to = NULL,
  maturities = NULL,
  cache = TRUE,
  cache_ttl_h = NULL
)
}
\arguments{
\item{curve}{Character. Which curve to fetch. One of \code{"nominal"},
\code{"real"}, \code{"inflation"}, \code{"ois"}, or \code{"blc"}. Defaults to
\code{"nominal"}. The commercial bank liability curve (\code{"blc"}) is only
published in the historical archive zip, so requests for it always
route through the archive path regardless of \code{from} / \code{to}.}

\item{measure}{Character. \code{"spot"} (default) or \code{"forward"}.}

\item{segment}{Character. \code{"standard"} (default) for the full maturity
spectrum in half-year steps, or \code{"short"} for the separately fitted
short end in monthly steps (one month to five years).}

\item{frequency}{Character. \code{"daily"} (default) or \code{"monthly"}. Monthly
archives are end-of-month observations and are much smaller files.}

\item{from, to}{Date or character ("YYYY-MM-DD"). Optional inclusive
bounds. When either is set, the function uses the BoE archive zip
(multi-decade history) and filters by date.}

\item{maturities}{Numeric vector of pillar maturities in years. When
\code{NULL} (default) a sensible set is chosen for the segment:
\code{c(0.5, 1, 2, 5, 10, 20)} for \code{"standard"} and
\code{c(0.5, 1, 2, 3, 5)} for \code{"short"}. Pillars not on the published grid
for the chosen curve and segment are dropped with a warning.}

\item{cache}{Logical. Use cached download if available and within the
TTL window (default \code{TRUE}).}

\item{cache_ttl_h}{Numeric. Cache time-to-live in hours. When \code{NULL}
(default) the TTL is 24 hours for the latest-month zip and 720 hours
(30 days) for archive zips.}
}
\value{
A \code{boe_tbl} data frame with columns \code{date} and one numeric
column per pillar named like \code{m0.5}, \code{m1}, \code{m2}, \code{m5}, \code{m10}, \code{m20}.
}
\description{
Convenience wrapper around \code{boe_curve()} that returns a wide-format
panel: one row per date, one column per requested pillar maturity.
This is the form most users want for time-series modelling and quick
plotting.
}
\details{
For each requested pillar, the function picks the published maturity
closest to the request (within a 0.05-year tolerance) and uses that.
The standard grid steps in 0.5-year increments and the short-end grid
(\code{segment = "short"}) in monthly increments, so pillars at integer,
half-integer, or whole-month maturities align exactly.
}
\examples{
\donttest{
if (requireNamespace("readxl", quietly = TRUE)) {
  op <- options(boe.cache_dir = tempdir())
  # Latest month: wide panel at chosen pillar maturities
  panel <- boe_curve_panel(curve = "nominal", measure = "spot",
                           maturities = c(2, 5, 10, 20))
  head(panel)
  options(op)
}
}

\dontrun{
# Historical panel (multi-decade archive download; not run automatically)
hist <- boe_curve_panel(curve = "nominal", measure = "spot",
                        from = "2020-01-01", maturities = c(2, 5, 10, 20))
}

}
\seealso{
Other interest rates: 
\code{\link{boe_bank_rate}()},
\code{\link{boe_curve}()},
\code{\link{boe_sonia}()},
\code{\link{boe_yield_curve}()}
}
\concept{interest rates}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_exchange_rate.R
\name{list_exchange_rates}
\alias{list_exchange_rates}
\title{List available exchange rate currencies}
\usage{
list_exchange_rates()
}
\value{
A data frame with columns:
\describe{
\item{currency}{Character. ISO currency code.}
\item{description}{Character. Currency name.}
\item{boe_code}{Character. BoE series code.}
}
}
\description{
Returns a data frame of currency codes and descriptions available
from the Bank of England exchange rate series.
}
\examples{
list_exchange_rates()

}
\seealso{
Other exchange rates: 
\code{\link{boe_exchange_rate}()}
}
\concept{exchange rates}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_tbl.R
\name{print.boe_tbl}
\alias{print.boe_tbl}
\title{Print method for boe_tbl}
\usage{
\method{print}{boe_tbl}(x, ...)
}
\arguments{
\item{x}{A \code{boe_tbl}.}

\item{...}{Passed to the underlying \code{print.data.frame} method.}
}
\value{
\code{x}, invisibly.
}
\description{
Adds a one-line provenance header above the data frame body. The
header summarises the request: number of series (and codes if few),
observation count, date range, frequency, and any vintage tag.
}
\examples{
\donttest{
op <- options(boe.cache_dir = tempdir())
x <- boe_bank_rate(from = "2020-01-01", frequency = "monthly")
print(x)
options(op)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_curve.R
\name{boe_curve}
\alias{boe_curve}
\title{Download BoE Anderson-Sleath fitted yield curves}
\source{
\url{https://www.bankofengland.co.uk/statistics/yield-curves}
}
\usage{
boe_curve(
  curve = c("nominal", "real", "inflation", "ois", "blc"),
  measure = c("spot", "forward"),
  segment = c("standard", "short"),
  frequency = c("daily", "monthly"),
  from = NULL,
  to = NULL,
  cache = TRUE,
  cache_ttl_h = NULL
)
}
\arguments{
\item{curve}{Character. Which curve to fetch. One of \code{"nominal"},
\code{"real"}, \code{"inflation"}, \code{"ois"}, or \code{"blc"}. Defaults to
\code{"nominal"}. The commercial bank liability curve (\code{"blc"}) is only
published in the historical archive zip, so requests for it always
route through the archive path regardless of \code{from} / \code{to}.}

\item{measure}{Character. \code{"spot"} (default) or \code{"forward"}.}

\item{segment}{Character. \code{"standard"} (default) for the full maturity
spectrum in half-year steps, or \code{"short"} for the separately fitted
short end in monthly steps (one month to five years).}

\item{frequency}{Character. \code{"daily"} (default) or \code{"monthly"}. Monthly
archives are end-of-month observations and are much smaller files.}

\item{from, to}{Date or character ("YYYY-MM-DD"). Optional inclusive
bounds. When either is set, the function uses the BoE archive zip
(multi-decade history) and filters by date.}

\item{cache}{Logical. Use cached download if available and within the
TTL window (default \code{TRUE}).}

\item{cache_ttl_h}{Numeric. Cache time-to-live in hours. When \code{NULL}
(default) the TTL is 24 hours for the latest-month zip and 720 hours
(30 days) for archive zips.}
}
\value{
A \code{boe_tbl} data frame with columns:
\describe{
\item{date}{Date. Observation date.}
\item{maturity_years}{Numeric. Maturity in years.}
\item{rate_pct}{Numeric. Yield or implied rate (percent).}
}
}
\description{
Downloads the Bank of England's published fitted yield curves at all
maturities (typically 0.5 to 25 or 40 years) using the Anderson and
Sleath (2001) smoothing methodology. Five curves are supported:
nominal gilt, real (index-linked) gilt, implied inflation, overnight
index swap (OIS), and the commercial bank liability curve (BLC).
}
\details{
Each curve is published in two segments. The default
\code{segment = "standard"} returns the full maturity spectrum in half-year
steps (0.5 years out to 25 or 40). \code{segment = "short"} returns the
short end of the curve in monthly steps (one month out to five years),
which the Bank fits separately and which is the segment most relevant
to near-term policy-rate and money-market analysis. The short end is
available for every curve in the latest month, and historically wherever
the BoE published it (e.g. OIS short-end data begins later than the OIS
standard curve); periods without a short-end sheet are skipped.

By default (\code{from = NULL}, \code{to = NULL}, \code{frequency = "daily"}) returns
the latest published month of daily data, matching the behaviour of
earlier releases of this package. Setting \code{from}, \code{to}, or \code{frequency}
switches to the BoE's full archive, which goes back to 1979 for nominal
gilts, 1985 for real, 2000 for BLC, and 2009 for OIS.

Requires the \pkg{readxl} package. Data is published as Excel workbooks
inside zip archives at
\url{https://www.bankofengland.co.uk/statistics/yield-curves}. Each
archive zip contains multiple per-period workbooks; this function
concatenates them transparently.
}
\examples{
\donttest{
if (requireNamespace("readxl", quietly = TRUE)) {
  op <- options(boe.cache_dir = tempdir())
  # Latest nominal spot curve at all maturities (default behaviour)
  curve <- boe_curve(curve = "nominal", measure = "spot")
  head(curve)

  # Short end of the nominal forward curve (monthly steps to 5 years)
  se <- boe_curve(curve = "nominal", measure = "forward",
                  segment = "short")
  range(se$maturity_years)
  options(op)
}
}

\dontrun{
# Historical archive: multi-decade downloads, so not run automatically.
# 10-year nominal spot back to 2010:
long <- boe_curve(curve = "nominal", from = "2010-01-01")

# End-of-month real curve since 1990:
real_m <- boe_curve(curve = "real", frequency = "monthly",
                    from = "1990-01-01")
}

}
\references{
Anderson, N. and Sleath, J. (2001). New estimates of the UK real
and nominal yield curves. \emph{Bank of England Working Paper No.
126.} \url{https://www.bankofengland.co.uk/working-paper/2001/new-estimates-of-the-uk-real-and-nominal-yield-curves}
}
\seealso{
Other interest rates: 
\code{\link{boe_bank_rate}()},
\code{\link{boe_curve_panel}()},
\code{\link{boe_sonia}()},
\code{\link{boe_yield_curve}()}
}
\concept{interest rates}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{boe_series}
\alias{boe_series}
\title{Catalogue of BoE series wrapped by this package}
\format{
A data frame with 8 columns:
\describe{
\item{code}{Character. BoE series code (e.g. \code{"IUDBEDR"}).}
\item{title}{Character. Human-readable description.}
\item{category}{Character. Topic grouping. One of \code{"interest_rates"},
\code{"exchange_rates"}, \code{"mortgage_market"}, \code{"consumer_credit"},
\code{"monetary_aggregates"}.}
\item{frequency}{Character. Native publication frequency
(\code{"daily"}, \code{"monthly"}, \code{"annual"}).}
\item{unit}{Character. Unit of measurement (\code{"percent"},
\code{"millions_gbp"}, \code{"currency_per_gbp"}, \code{"index"}, \code{"count"}).}
\item{start_date}{Date. Earliest available observation date.}
\item{seasonal_adjustment}{Character or \code{NA}. \code{"SA"}, \code{"NSA"}, or
\code{NA} if not applicable.}
\item{helper}{Character. Convenience function in the package that
wraps the series.}
}
}
\source{
\url{https://www.bankofengland.co.uk/boeapps/database/}
}
\usage{
boe_series
}
\description{
A reference data frame of Bank of England Statistical Database series
codes for which the package provides a named convenience function.
Used by \code{\link[=boe_search]{boe_search()}} and \code{\link[=boe_browse]{boe_browse()}}.
}
\examples{
head(boe_series)
table(boe_series$category)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe-package.R
\docType{package}
\name{boe-package}
\alias{boe}
\alias{boe-package}
\title{boe: Download Data from the 'Bank of England' Statistical Database}
\description{
Provides functions to download and tidy statistical data published by the 'Bank of England' \url{https://www.bankofengland.co.uk}. Covers Bank Rate, 'SONIA', gilt yields, exchange rates, mortgage rates, mortgage approvals, consumer credit, and money supply. Series are fetched from the 'Bank of England Interactive Statistical Database' using its CSV endpoint. Data is cached locally between sessions.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://charlescoverdale.github.io/boe/}
  \item \url{https://github.com/charlescoverdale/boe}
  \item Report bugs at \url{https://github.com/charlescoverdale/boe/issues}
}

}
\author{
\strong{Maintainer}: Charles Coverdale \email{charlesfcoverdale@gmail.com}

}
\concept{Bank of England}
\concept{SONIA}
\concept{UK central bank}
\concept{exchange rates}
\concept{interest rates}
\concept{monetary policy}
\concept{money supply}
\concept{mortgage rates}
\concept{yield curve}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_yield_curve.R
\name{boe_yield_curve}
\alias{boe_yield_curve}
\title{Download UK gilt yields}
\source{
\url{https://www.bankofengland.co.uk/boeapps/database/}
}
\usage{
boe_yield_curve(
  from = "2000-01-01",
  to = Sys.Date(),
  maturity = c("5yr", "10yr", "20yr"),
  type = c("nominal", "real"),
  measure = c("par_yield", "zero_coupon"),
  cache = TRUE
)
}
\arguments{
\item{from}{Date or character (YYYY-MM-DD). Start date.}

\item{to}{Date or character (YYYY-MM-DD). End date. Defaults to today.}

\item{maturity}{Character vector. One or more of \code{"5yr"}, \code{"10yr"},
\code{"20yr"}. Defaults to all three.}

\item{type}{Character. \code{"nominal"} (default) or \code{"real"}.}

\item{measure}{Character. \code{"par_yield"} (default, nominal only) or
\code{"zero_coupon"}.}

\item{cache}{Logical. Use cached data if available (default \code{TRUE}).}
}
\value{
A data frame with columns:
\describe{
\item{date}{Date. Observation date.}
\item{maturity}{Character. Maturity label (e.g. \code{"5yr"}).}
\item{yield_pct}{Numeric. Yield (percent).}
}
}
\description{
Downloads nominal or real gilt yields at specified maturities from the
Bank of England yield curve data. Nominal par yields are available
daily from late 1993; real zero-coupon yields from 1985.
}
\examples{
\donttest{
op <- options(boe.cache_dir = tempdir())
# 10-year nominal gilt yield since 2020
boe_yield_curve(from = "2020-01-01", maturity = "10yr")

# Full nominal curve
boe_yield_curve(from = "2020-01-01")

# Real yields
boe_yield_curve(from = "2020-01-01", type = "real", measure = "zero_coupon")
options(op)
}

}
\seealso{
Other interest rates: 
\code{\link{boe_bank_rate}()},
\code{\link{boe_curve}()},
\code{\link{boe_curve_panel}()},
\code{\link{boe_sonia}()}
}
\concept{interest rates}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_money_supply.R
\name{boe_money_supply}
\alias{boe_money_supply}
\title{Download M4 money supply}
\source{
\url{https://www.bankofengland.co.uk/boeapps/database/}
}
\usage{
boe_money_supply(
  from = "1982-06-01",
  to = Sys.Date(),
  seasonally_adjusted = TRUE,
  cache = TRUE
)
}
\arguments{
\item{from}{Date or character (YYYY-MM-DD). Start date. Defaults to
\code{"1982-06-01"}.}

\item{to}{Date or character (YYYY-MM-DD). End date. Defaults to today.}

\item{seasonally_adjusted}{Logical. Return seasonally adjusted series
(default \code{TRUE}) or non-seasonally adjusted (\code{FALSE}).}

\item{cache}{Logical. Use cached data if available (default \code{TRUE}).}
}
\value{
A data frame with columns:
\describe{
\item{date}{Date. End of month.}
\item{amount_gbp_m}{Numeric. M4 amounts outstanding (millions of
pounds).}
}
}
\description{
Downloads monthly M4 (broad money) amounts outstanding from the Bank
of England. Available from June 1982.
}
\examples{
\donttest{
op <- options(boe.cache_dir = tempdir())
boe_money_supply(from = "2000-01-01")
options(op)
}

}
\concept{money}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_mortgage_rates.R
\name{boe_mortgage_rates}
\alias{boe_mortgage_rates}
\title{Download quoted mortgage interest rates}
\source{
\url{https://www.bankofengland.co.uk/boeapps/database/}
}
\usage{
boe_mortgage_rates(
  type = c("2yr_fixed", "3yr_fixed", "5yr_fixed", "svr"),
  from = "1995-01-01",
  to = Sys.Date(),
  cache = TRUE
)
}
\arguments{
\item{type}{Character vector. One or more of \code{"2yr_fixed"},
\code{"3yr_fixed"}, \code{"5yr_fixed"}, \code{"svr"}. Defaults to all four.}

\item{from}{Date or character (YYYY-MM-DD). Start date. Defaults to
\code{"1995-01-01"}.}

\item{to}{Date or character (YYYY-MM-DD). End date. Defaults to today.}

\item{cache}{Logical. Use cached data if available (default \code{TRUE}).}
}
\value{
A data frame with columns:
\describe{
\item{date}{Date. End of month.}
\item{type}{Character. Mortgage product type.}
\item{rate_pct}{Numeric. Quoted rate (percent).}
}
}
\description{
Downloads monthly quoted (advertised) mortgage rates from the Bank of
England, including fixed-rate products and the standard variable rate
(SVR). Available from January 1995.
}
\examples{
\donttest{
op <- options(boe.cache_dir = tempdir())
# All mortgage rate types since 2015
boe_mortgage_rates(from = "2015-01-01")

# 2-year fixed only
boe_mortgage_rates(type = "2yr_fixed", from = "2020-01-01")
options(op)
}

}
\seealso{
Other credit and housing: 
\code{\link{boe_consumer_credit}()},
\code{\link{boe_mortgage_approvals}()}
}
\concept{credit and housing}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_mpr_forecasts.R
\name{boe_mpr_forecasts}
\alias{boe_mpr_forecasts}
\title{Monetary Policy Report forecast data}
\source{
\url{https://www.bankofengland.co.uk/monetary-policy}
}
\usage{
boe_mpr_forecasts(
  series = c("cpi_inflation", "gdp_growth", "gdp_level", "unemployment", "bank_rate"),
  month = NULL,
  year = NULL,
  cache = TRUE
)
}
\arguments{
\item{series}{Character vector. One or more of:
\code{"cpi_inflation"}, \code{"gdp_growth"}, \code{"gdp_level"},
\code{"unemployment"}, \code{"bank_rate"}. Defaults to all five.}

\item{month}{Character. Publication month of the report, e.g.
\code{"february"} or \code{"may"}. The report is published roughly quarterly,
but the exact month drifts between years (for example, the second
2026 report appeared in April, not May), so any month name is
accepted and its existence is verified against the Bank's website.
Supply with \code{year}. If both are \code{NULL}, the most recent compatible
release is selected automatically.}

\item{year}{Integer. MPR year, 2019 or later. Supply with \code{month}.
If both are \code{NULL}, the most recent compatible release is selected
automatically.}

\item{cache}{Logical. Use cached download if available (default
\code{TRUE}). Older releases never change so the cache never expires;
the latest release is refreshed if older than 24 hours.}
}
\value{
A \code{boe_tbl} data frame with columns:
\describe{
\item{date}{Date. Publication date of the MPR (start of quarter
the report covers).}
\item{horizon}{Character. Forecast horizon label (e.g. \code{"2026 Q1"}).}
\item{horizon_date}{Date. Start of the forecast quarter.}
\item{series}{Character. Series identifier (e.g. \code{"cpi_inflation"}).}
\item{value}{Numeric. Forecast value (percent for rates and
growth; index for \code{gdp_level}).}
}
}
\description{
Downloads the Bank of England's Monetary Policy Report (MPR) and
parses headline projections from the Projections Databank workbook.
Returns a long-format data frame where each row is one (publication
date, forecast horizon, series) triple.
}
\details{
Coverage runs quarterly from November 2019 (when the report was
renamed from Inflation Report) to the latest published release.

Requires the \pkg{readxl} package. The MPR is published as a zip
archive containing a Projections Databank workbook plus chart data
and slides; this function only reads the projection sheets.

Each row of a projection sheet is one MPR publication; columns are
forecast quarters. The same publication therefore contributes
multiple rows here, one per forecast horizon.
}
\section{Release format and automatic fallback}{

From the April 2026 report the Bank moved to a scenario-based
"Scenario Projections Databank" with a transposed layout (following
the Bernanke review of forecasting). That format is not parsed by
this function yet. When automatic selection encounters such a
release it skips it, falls back to the most recent compatible release
(the classic "Projections Databank" workbook), and warns. Requesting
a scenario-format release explicitly via \code{month}/\code{year} raises a
clear error. Pre-2020 MPRs that predate the single "Projections
Databank" workbook may also error.
}

\examples{
\donttest{
if (requireNamespace("readxl", quietly = TRUE)) {
  op <- options(boe.cache_dir = tempdir())

  # Latest CPI inflation projections
  cpi <- boe_mpr_forecasts(series = "cpi_inflation")
  head(cpi)

  options(op)
}
}

}
\seealso{
\code{\link[=boe_mpc_decisions]{boe_mpc_decisions()}}, \code{\link[=boe_mpc_votes]{boe_mpc_votes()}}

Other monetary policy: 
\code{\link{boe_mpc_decisions}()},
\code{\link{boe_mpc_votes}()}
}
\concept{monetary policy}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_get.R
\name{boe_get}
\alias{boe_get}
\title{Fetch any series from the Bank of England Statistical Database}
\source{
\url{https://www.bankofengland.co.uk/boeapps/database/}
}
\usage{
boe_get(series_codes, from = "1960-01-01", to = Sys.Date(), cache = TRUE)
}
\arguments{
\item{series_codes}{Character vector of one or more BoE series codes.}

\item{from}{Date or character (YYYY-MM-DD). Start date. Defaults to
\code{"1960-01-01"}.}

\item{to}{Date or character (YYYY-MM-DD). End date. Defaults to today.}

\item{cache}{Logical. Use cached data if available (default \code{TRUE}).}
}
\value{
A data frame with columns:
\describe{
\item{date}{Date. Observation date.}
\item{code}{Character. BoE series code.}
\item{value}{Numeric. Observation value.}
}
}
\description{
The core data retrieval function. Fetches one or more series by their
BoE series codes and returns a tidy data frame. Use this when the
convenience functions (e.g. \code{\link[=boe_bank_rate]{boe_bank_rate()}}, \code{\link[=boe_exchange_rate]{boe_exchange_rate()}})
do not cover the series you need.
}
\details{
Series codes can be found via the Bank of England Interactive
Statistical Database at
\url{https://www.bankofengland.co.uk/boeapps/database/}.
}
\examples{
\donttest{
op <- options(boe.cache_dir = tempdir())
# Bank Rate since 2000
boe_get("IUDBEDR", from = "2000-01-01")

# Multiple series
boe_get(c("IUDBEDR", "IUDSOIA"), from = "2020-01-01")
options(op)
}

}
\seealso{
Other data access: 
\code{\link{clear_cache}()}
}
\concept{data access}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_mpc_decisions.R
\name{boe_mpc_decisions}
\alias{boe_mpc_decisions}
\title{Bank Rate decision history}
\source{
Derived from BoE series \code{IUDBEDR} (daily Bank Rate). See
\url{https://www.bankofengland.co.uk/monetary-policy}.
}
\usage{
boe_mpc_decisions(from = "1997-06-06", to = Sys.Date(), cache = TRUE)
}
\arguments{
\item{from}{Date or character. Start date. Defaults to
\code{"1997-06-06"} (the first MPC meeting).}

\item{to}{Date or character. End date. Defaults to today.}

\item{cache}{Logical. Use cached Bank Rate data if available
(default \code{TRUE}).}
}
\value{
A \code{boe_tbl} data frame with columns:
\describe{
\item{date}{Date. Effective date of the rate change.}
\item{new_rate_pct}{Numeric. Bank Rate after the decision (percent).}
\item{prev_rate_pct}{Numeric. Bank Rate before the decision (percent).}
\item{change_bps}{Integer. Change in basis points (positive = hike,
negative = cut).}
\item{direction}{Character. \code{"hike"} or \code{"cut"}.}
}
}
\description{
Returns the history of Monetary Policy Committee decisions to change
Bank Rate, derived from the daily Bank Rate series. Each row is one
rate-change event, showing the effective date, the new rate, the
previous rate, and the change in basis points. Holds (meetings where
the rate was unchanged) are not included; for the full meeting-level
record including holds, see \code{\link[=boe_mpc_votes]{boe_mpc_votes()}}.
}
\examples{
\donttest{
op <- options(boe.cache_dir = tempdir())
# All MPC decisions since the global financial crisis
boe_mpc_decisions(from = "2007-01-01")

# Just decisions in 2024 to date
boe_mpc_decisions(from = "2024-01-01")
options(op)
}

}
\seealso{
\code{\link[=boe_bank_rate]{boe_bank_rate()}}, \code{\link[=boe_mpc_votes]{boe_mpc_votes()}}

Other monetary policy: 
\code{\link{boe_mpc_votes}()},
\code{\link{boe_mpr_forecasts}()}
}
\concept{monetary policy}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boe_search.R
\name{boe_search}
\alias{boe_search}
\title{Search the BoE series catalogue}
\usage{
boe_search(query = NULL, category = NULL, frequency = NULL)
}
\arguments{
\item{query}{Character. Keyword(s) to match against the \code{title} and
\code{code} columns. Case-insensitive substring match. If \code{NULL},
returns all rows (subject to other filters).}

\item{category}{Character. Optional filter on the \code{category} column,
one or more of \code{"interest_rates"}, \code{"exchange_rates"},
\code{"mortgage_market"}, \code{"consumer_credit"}, \code{"monetary_aggregates"}.}

\item{frequency}{Character. Optional filter on the \code{frequency}
column, one or more of \code{"daily"}, \code{"monthly"}, \code{"annual"}.}
}
\value{
A data frame with the same columns as \link{boe_series},
restricted to matching rows.
}
\description{
Filters the \link{boe_series} catalogue by keyword and optional category
and frequency. Useful for finding a series code without leaving R.
Equivalent to a \code{grepl} against the title and code columns.
}
\examples{
# Find mortgage-related series
boe_search("mortgage")

# All daily interest-rate series
boe_search(category = "interest_rates", frequency = "daily")

# Locate the Bank Rate code
boe_search("bank rate")

}
\seealso{
\code{\link[=boe_browse]{boe_browse()}}, \link{boe_series}

Other discovery: 
\code{\link{boe_browse}()}
}
\concept{discovery}
